C=======================================================================
C
C     PROGRAM RELAX
C     VERSION 90-1 (DECEMBER 1990)
C     VERSION 91-1 (NOVEMBER 1991)*UPDATED TO PERFORM CALCULATIONS 2
C                                  SEPARATE WAYS AND INSURE COMPLETE
C                                  NUMBER OF PARTICLES AND ENERGY
C                                  BALANCE.
C     VERSION 91-2 (MAY 1992)     *ADDED SUM OVER SUBSHELL TO DEFINE
C                                  EQUIVALENT SHELL RESULTS
C     VERSION 2014 (Sept. 2014)  *Updated for ENDLX Format
C                  (Oct.  2014)  *Changed from D to E exponential form
C                                 for OUTPUT to improve compatibility.
C
C     Written by Dermott E. Cullen
C                1466 Hudson Way
C                Livermore, CA 94550
C                U.S.A.
C     Telephone  925-443-1911
C     E.Mail     RedCullen1@comcast.net
C     Website    http://home.comcast.net/~redcullen1
C
C     PURPOSE
C     ==================================================================
C     THIS PROGRAM IS DESIGNED TO CALCULATE ATOM RELAXATION SPECTRA OF
C     X-RAYS AND ELECTRONS DUE TO BOUND-BOUND TRANSITIONS. THIS
C     CALCULATION IS BASED ON THE ATOM TRANSITION DATA CONTAINED IN
C     THE LIVERMORE EVALUATED ATOMIC DATA LIBRARY (EADL).
C
C     THE PROGRAM WILL ALSO CALCULATE THE SPECTRUM OF FREE-BOUND
C     TRANSITIONS ASSUMING THAT THE ATOM WILL RELAX BACK TO IT NEUTRAL
C     GROUND STATE BY FILLING ALL REMAINING ELECTRON HOLES THROUGH
C     TRANSITIONS CAPTURING ELECTRONS DIRECTLY FROM THE CONTINUUM.
C
C     THIS PROGRAM WILL ALSO CALCULATE SUBSHELL AVERAGED VALUES, SUCH
C     AS TOTAL ENERGY TO X-RAYS, ELECTRONS AND FREE-BOUND, AVERAGE
C     NUMBER OF EMITTED X-RAYS, ELECTRONS AND FREE-BOUND TRANSITIONS
C     AND AVERAGE ENERGY OF X-RAYS, ELECTRONS AND FREE-BOUND TRANSITIONS
C
C     THE SUM OF THE TOTAL ENERGY TO X-RAYS, ELECTRONS AND FREE-BOUND
C     IS EQUAL TO THE BINDING ENERGY OF THE SUBSHELL IN WHICH THE
C     INITIAL VACANCY (HOLE) IS LOCATED. THE DIVISION OF ENERGY BETWEEN
C     X-RAYS, ELECTRONS AND FREE-BOUND IS COMPLETELY DEFINED BY THE
C     DATA IN EADL. HOWEVER, THE SPECTRUM OF FREE-BOUND TRANSITIONS
C     IS NOT COMPLETELY DEFINED AND THE ASSUMPTION THAT IS USED HERE
C     WILL CONSERVE THE TOTAL ENERGY OF THE SUM OF ALL FREE-BOUND
C     TRANSITIONS AND PRODUCE THE HARDEST POSSIBLE SPECTRUM WITH THE
C     FEWEST POSSIBLE EMITTED X-RAYS.
C
C     DESCRIPTION OF FINAL ATOM
C     ==================================================================
C     THIS PROGRAM CAN CALCULATE RESULTS FOR A FINAL ATOM,
C
C     1) NEUTRAL  - AS ELECTRONS ARE EJECTED FROM THE ATOMS, HOLES CAN
C                   BE FILLED BY ELECTRONS FROM THE CONTINUUM (OUTSIDE
C                   THE ATOM). IN THIS CASE TRANSITIONS CAN ALWAYS
C                   OCCUR, SINCE WE ASSUME THAT AS ELECTRONS LEAVE A
C                   SUBSHELL THEY ARE REPLACED BY ELECTRONS UNDERGOING
C                   TRANSITIONS FROM ANOTHER SUBSHELL OR THE CONTINUUM,
C                   I.E., WE HAVE AN INFINITE SUPPLY OF ELECTRONS
C                   AVAILABLE. IN THIS CASE THE SET OF EQUATIONS HAS A
C                   UNIQUE SOLUTION.
C
C     2) ISOLATED - HOLES CANNOT BE FILLED BY ELECTRONS FROM THE
C                   CONTINUUM. IN THIS CASE TRANSITIONS BETWEEN
C                   SUBSHELLS CAN ONLY OCCUR IF THERE ARE REMAINING
C                   ELECTRONS IN THE SUBSHELL FROM WHICH THE ELECTRON
C                   IS UNDERGOING A TRANSITION TO FILL A HOLE. IN THIS
C                   CASE THERE WILL BE COMPETITION BETWEEN THE VARIOUS
C                   SUBSHELLS FOR ELECTRON TRANSITIONS FROM ONE SUBSHELL
C                   TO FILL A HOLE IN ANOTHER SUBSHELL - IN THIS CASE
C                   THE SET OF EQUATIONS DOES NOT HAVE A UNIQUE SOLUTION
C                   - THIS PROGRAM WILL CALCULATE THE SOLUTION THAT
C                   CORRESPONDS TO THE MAXIMUM ENERGY RADIATED FROM THE
C                   ATOM BY THE MINIMUM NUMBER OF X-RAYS AND ELECTRON
C                   BY GIVING PRIORITY TO INNER SUBSHELL TRANSITIONS
C                   OVER OUTER SUBSHELLS TO DEFINE THE COMPETITION.
C
C     ASSUMPTIONS
C     ==================================================================
C     1) WE ASSUME THAT THE BINDING ENERGY OF THE NEUTRAL ATOMS IN THEIR
C        GROUND STATE IS THE SAME AS THE BINDING ENERGY IN ANY IONIZED
C        STATE - THIS ASSUMPTION IS BASED ON THE FACT THAT AS YET OUR
C        DATA BASE - EADL - ONLY CONTAINS A SINGLE BINDING ENERGY TO
C        DESCRIBE EACH SUBSHELL - AT THE CURRENT TIME IT IS BEYOND OUR
C        CAPABILITIES TO PROVIDE MORE EXTENSIVE SETS OF BINDING ENERGY
C        TO DESCRIBE ALL COMBINATIONS OF IONIZED STATES OF EACH ATOM.
C
C     2) WE ASSUME AN INITIAL CONDITION OF ONE SUBSHELL OF ONE ELEMENT
C        TO BE SINGLY IONIZED AND THEN DESCRIBE THE RELAXATION DUE TO
C        BOUND-BOUND TRANSITIONS. WE DO NOT CONSIDER EXCITED ATOMS AND
C        EADL DOES NOT UNIQUELY DEFINE FREE-BOUND TRANSITIONS.
C
C     3) WE ASSUME THAT THE RADIATIVE AND NON-RADIATIVE TRANSITION
C        DATA IS IN UPPER TRIANGLAR FORM BASED ON ASCENDING SUBSHELL
C        INDEX - I.E., NO TRANSITIONS BETWEEN SUBSHELL I AND SUBSHELL
C        K IF I IS GREATER THAN OR EQUAL TO K. THIS CONDITION IS
C        SATIFIED BY THE ENTIRE EADL DATA BASE AS OF DECEMBER 1990.
C
C     ENERGY OF EMITTED X-RAYS AND ELECTRONS
C     ==================================================================
C     FOR A RADIATIVE TRANSITION BETWEEN SUBSHELLS I AND J THE ENERGY
C     OF THE EMITTED X-RAY IS DEFINED TO BE,
C
C     E(X-RAY)    = B.E.(I) - B.E.(J)
C
C     FOR A NON-RADIATIVE TRANSITION BETWEEN SUBSHELLS I AND J IN WHICH
C     AN ELECTRON IS EMITTED FROM SUBSHELL K THE ENERGY OF THE EMITTED
C     ELECTRON IS DEFINED TO BE,
C
C     E(ELECTRON) = B.E.(I) - (B.E.(J) + B.E.(K))
C
C     TRANSITION PROBABILITIES
C     ==================================================================
C     TRANSISTION PROBABILITIES ARE DEFINED BY 2 MATRICES
C
C     TR(I,J)    = RADIATIVE TRANSISTIONS    - AN ELECTRON TRANSITION
C                  J ---> I, ACCOMPANIED BY THE EMISSION OF A PHOTON,
C                  FILLING A VACANCY IN I, BUT CREATING A VACANCY IN J.
C     TNR(I,J,K) = NON-RADIATIVE TRANSITIONS - AN ELECTRON TRANSITION
C                  J ---> I, ACCOMPANIED BY THE EMISSION OF AN
C                  ELECTRON FROM K, FILLING A VACANCY IN I, BUT
C                  CREATING VACANCIES IN BOTH J AND K.
C
C     WHERE I, J, K REFER TO ATOM SUBSHELLS
C
C     THESE 2 MATRICES ARE DERIVED BY THIS CODE DIRECTLY FROM THE
C     EADL TRANSITION PROBABILITIES - INSURING EXACT NORMALIZATION
C     IN ORDER TO RETURN AN ATOM TO NEUTRALITY,
C
C     1 = SUM (OVER J) TR(I,J) + SUM (OVER J AND K) TNR(I,J,K)
C
C     I.E., ANY HOLE WILL ALWAYS BE EXACTLY FILLED.
C
C     THEREFORE THIS PROGRAM IS MERELY SOLVING A SET OF COUPLED MATRIX
C     EQUATIONS TO DEFINED THE EMITTED PHOTON AND ELECTRON SPECTRA.
C
C     FOR A SINGLE VACANCY IN SUBSHELL I, TR(I,J) AND TNR(I,J,K)
C     DEFINE BOTH WHAT IS EMITTED AS THE HOLE IN I IS FILLED - AND
C     WHAT HOLES WILL RESULT IN J AND K. THEREFORE STARTING FROM I
C     THIS CODE MUST SOLVE FOR ALL OF THE TRANSITIONS - BOTH DUE TO
C     PRIMARY EMISSION AS THE INITIAL VACANCY IS FILLED - AND ALL
C     SECONDARY HOLES (VACANCIES) THAT ARE CREATED DUE TO PRIMARY
C     TRANSITIONS. THIS IS ACCOMPLISHED BY STARTING WITH ONLY A SINGLE
C     VACANCY IN I, KEEPING TRACK OF ALL VACANCIES GENERATED IN OTHER
C     SUBSHELLS, AND WORKING FROM INNER (HIGHER ENERGY) SUBSHELL TO
C     THE OUTER MOST (LOWEST ENERGY) SUBSHELL, OF AN ATOM.
C
C     UNCERTAINTY IN RESULTS
C     ==================================================================
C     SINCE THERE IS AN UNCERTAINTY ASSOCIATED WITH EACH BINDING ENERGY
C     (B.E.) THERE WILL ALSO BE AN UNCERTAINTY ASSOCIATED WITH THE
C     ENERGY OF THE EMITTED X-RAYS AND ELECTRONS. THE BINDING ENERGIES
C     FOR GROUND STATE NEUTRAL ATOMS IS ABOUT 1 PER-CENT AND IS MORE
C     FOR IONIZED ATOMS. THIS IMPLIES THAT FOR TRANSITIONS WHERE THE
C     BINDING ENERGIES ARE ALMOST EQUAL THERE WILL BE A RATHER LARGE
C     UNCERTAINTY IN THE EMISSION ENERGY. GENERALLY THIS OCCURS FOR
C     LOW ENERGY X-RAYS AND ELECTRONS, WHERE THE EMISSION ENERGY IS
C     MUCH LESS THAN ANY OF THE BINDING ENERGIES INVOLVED IN THE
C     TRANSITION, E.G., FOR RADIATIVE, B.E.(K) ALMOST EQUAL TO B.E.(I),
C     FOR NON-RADIATIVE, B.E.(K) ALMOST EQUAL TO B.E.(I)+B.E.(J).
C
C     NEGATIVE ENERGY ELECTRONS AND ENERGY RE-BALANCE
C     ==================================================================
C     THE UNCERTAINTY IN THE BINDING ENERGY CAN RESULT IN TRANSITIONS
C     IN WHICH THE CALCULATED ENERGY OF EMITTED ELECTRONS CAN BE
C     NEGATIVE, I.E. B.E.(I)+B.E.(J) EXCEEDS B.E.(K). IN THIS CASE
C     THE TRANSITION CANNOT BE SIMPLY IGNORED, SINCE THE CREATION OF
C     HOLES DUE TO THESE TRANSITIONS CAN BE VERY IMPORTANT IN DEFINING
C     THE NUMBER OF EMITTED X-RAYS AND ELECTRONS.
C
C     IN THIS CODE WHEN THE CALCULATED ENERGY OF AN EMITTED ELECTRON
C     IS NEGATIVE THE TRANSITION IS ALLOWED TO PROCEED AND THE ELECTRON
C     IS ASSIGNED AN ENERGY OF EXACTLY ZERO. THIS PROCEDURE HAS BEEN
C     FOUND TO IN ALL CASES BE CONSISTENT WITH THE ASSUMED UNCERTAINTY
C     IN THE BINDING ENERGIES, I.E., A CALCULATED NEGATIVE ELECTRON
C     ENERGY REALLY INDICATES THE EMISSION OF A VERY LOW ENERGY ELECTRON
C     WITH A LARGE UNCERTAINTY IN ITS ENERGY - WHICH WE HERE DEFINE TO
C     BE A ZERO ENERGY ELECTRON.
C
C     THIS PROCEDURE CAN RESULT IN THE LOSS OF AN ENERGY BALANCE. THE
C     TOTAL ENERGY TO X-RAYS, ELECTRONS CAN BE DEFINED BY SUMMING OVER
C     ALL TRANSITIONS THE PRODUCT OF THE EMISSION ENERGY AND THE
C     TRANSITION PROBABILITY. SIMILARLY THE TOTAL ENERGY TO LOCAL (FREE-
C     BOUND) CAN BE DEFINED BY SUMMING OVER ALL ELECTRON HOLES THAT
C     REMAIN = THE PRODUCT OF THE BINDING ENERGY AND THE NUMBER OF
C     HOLES IN EACH SUBSHELL. THIS PROCEDURE RESULTS IN AN EXACT ENERGY
C     BALANCE, EXCEPT IN THE CASE WHERE WE HAVE ASSIGNED ZERO ENERGY
C     TO ELECTRONS WHOSE CALCULATED EMISSION ENERGY IS NEGATIVE.
C
C     IN THE CASE WHEN WE ASSIGN ZERO ENERGY TO ELECTRONS THE RESULT
C     IS THAT THE CACULATED ENERGY BALANCE AS THE SUM OF THE TOTAL
C     X-RAY, ELECTRON AND LOCAL ENERGIES EXCEEDS THE BINDING ENERGY,
C     I.E., WE DO NOT HAVE AN ENERGY BALANCE.
C
C     TO ACHIEVE AN ENERGY BALANCE WE CALCULATE THE TOTAL ENERGY TO
C     X-RAYS AND ELECTRONS AND DEFINE THE LOCAL ENERGY TO BE THE
C     DIFFERENCE BETWEEN THIS SUM AND THE BINDING ENERGY. IN ORDER
C     TO HAVE THE CALCULATED LOCAL ENERGY BASED ON REMAINING HOLES
C     AND BINDING ENERGIES BE EQUAL TO THE CALCULATED LOCAL ENERGY
C     BASED ON THE BINDING ENERGY MINUS THE SUM OF THE TOTAL ENERGY
C     TO X-RAYS AND ELECTRONS, WE DEFINE THE ENERGY OF FREE-BOUND
C     TRANSITIONS TO BE SLIGHTLY LESS THAN THE BINDING ENERGY, BY
C     AN AMOUNT CORRESPONDING TO THE RATIO OF THE LOCAL ENERGY
C     CALCULATED BY THE TWO METHODS - THUS FORCING AN ENERGY BALANCE.
C
C     GENERALLY WHEN NEGATIVE ENERGY ELECTRONS OCCUR THE SUM OF
C     THEIR ENERGIES IS ONLY A SMALL FRACTION OF 1 PER-CENT OF THE
C     BINDING ENERGY, I.E., MUCH LESS THAN THE UNCERTAINTY IN THE
C     BINDING ENERGIES. THEREFORE THIS RE-BALANCE PROCEDURE WILL
C     GENERALLY SHIFT THE FREE-BOUND EMISSION ENERGIES BY ONLY A
C     SMALL FRACTION OF 1 PER-CENT, MUCH LESS THAN THE UNCERTAINTY
C     IN THE BINDING ENERGY AND AS SUCH OF NO PRACTICAL CONCERN.
C
C     NOTE - THIS RE-BALANCE SHOULD NEVER TO ACHIEVED BY CHANGING
C     THE NUMBER OF REMAINING HOLES IN EACH SUBSHELL BECAUSE THESE
C     HOLES ARE REQUIRED TO EVENTUALLY RETURN THE ATOM THE EXACT
C     NEUTRALITY.
C
C     THIS ENERGY RE-BALANCE PROCEDURE, WHICH IS ONLY NECESSARY WHEN
C     USING THE BINDING ENERGIES WOULD LEAD TO THE EMISSION OF A
C     NON-PHYSICAL NEGATIVE ENERGY ELECTRON, IS CONSISTENT WITH THE
C     KNOWN UNCERTAINTY IN THE BINDING ENERGIES.
C
C     INPUT ENDL FORMAT
C     ==================================================================
C     THIS PROGRAM ONLY USES THE ENDL CHARACTER OR CARD IMAGE FORMAT
C     (AS OPPOSED TO THE BINARY FORMAT). THE SOURCE OF ALL EVALUATED
C     DATA USED BY THIS PROGRAM IS THE LIVERMORE EVALUATED ATOMIC DATA
C     LIBRARY (EADL).
C
C     PROGRAM OPERATION
C     ==================================================================
C     BY INPUT PARAMETERS THE USER FIRST SELECTS OUTPUT OPTIONS (SEE,
C     DEFINITIONS BELOW). NEXT THE USER SPECIFIES A RANGE OF ELEMENTS
C     AND SUBSHELLS TO PROCESS.
C
C     THE SPECTRA FOR EACH REQUESTED ELEMENT/SUBSHELL ARE CALCULATED
C     SEPARATELY AND ARE ONLY AVAILABLE IN CORE BEFORE THE SPECTRA FOR
C     THE NEXT ELEMENT/SUBSHELL ARE CALCULATED. SUBSHELL AVERAGED
C     PARAMETERS ARE ACCUMULATED FOR EACH ELEMENT SEPARATELY AND ARE
C     ONLY AVAILABLE IN CORE BEFORE CALCULATIONS START ON THE NEXT
C     ELEMENT.
C
C     ONCE THE ENTIRE USER SPECIFIED RANGE OF ELEMENTS AND SUBSHELLS
C     HAVE BEEN PROCESSD THE PROGRAM WILL ATTEMPT TO READ THE NEXT
C     USER SPECIFIED RANGE OF ELEMENTS AND SUBSHELLS. CALCULATIONS
C     WILL CONTINUE UNTIL AN END OF FILE IS READ ON THE INPUT FILE.
C
C     OUTPUT OPTIONS
C     ==================================================================
C     THIS PROGRAM CONTAINS INPUT OPTIONS TO PRODUCE THE FOLLOWING
C     OUTPUT,
C     1) A LISTING - A SUMMARY OF EACH SUBSHELL AND THE SPECTRA
C     2) ENDL      - CALCULATE SUBSHELL AVERAGED VALUES AND NORMALIZED
C                    TRANSITION DATA IN THE ENDL FORMAT
C     3) PLOTTAB   - SPECTRA IN THE PLOTTAB FORMAT WHICH CAN BE USED
C                    WITH PROGRAM PLOTTAB TO OBTAIN PLOTS OF THE
C                    RESULTS.
C
C     USER INTERFACE
C     ==================================================================
C     IF YOU WISH TO OBTAIN RESULTS IN ANY OTHER FORM YOU MAY DO SO
C     BY INSERTING CODING INTO SUBROUTINE USER1 AND/OR USER2. USER1
C     IS CALLED AFTER THE SPECTRA HAVE BEEN CALCULATED FOR EACH
C     SUBSHELL - IF YOU WANT THE SPECTRA THIS IS THE TIME AND PLACE
C     TO GET THEM. USER2 IS CALLED AFTER ALL RESULTS HAVE BEEN
C     CALCULATED FOR EACH ELEMENT - IF YOU WANT SUBSHELL AVERAGED
C     RESULTS THIS IS THE TIME AND PLACE TO GET THEM.
C
C     ALL VARIABLES OF INTEREST TO THE USERS ARE DESCRIBED BELOW.
C
C     I/O FILES
C     ==================================================================
C     INPUT FILES
C     -----------
C     UNIT  FILENAME     DESCRIPTION
C     ----  --------     -----------
C       5   RELAX.INP    INPUT PARAMETERS
C      10   ENDL.IN      ENDL FORMATTED DATA
C
C     OUTPUT FILES
C     -----------
C     UNIT  FILENAME          DESCRIPTION
C     ----  --------          -----------
C       6   RELAX.LST         OUTPUT LISTING
C      11   ENDL.OUT          ENDL FORMATTED DATA
C      12   PLOTTAB.CUR       NORMALIZED INTEGRAL OF SPECTRA
C                             FLUORESCENCE YIELD VS. Z BY SUBSHELL
C      14   PLOTTAB.PNT       SPECTRA POINT VALUES
C      15   PLOTTAB.INP       PLOTTAB INPUT PARAMETERS
C      17   PLOTTAB.CUR.SHELL PLOTTAB OUTPUT SUMMED OVER SHELLS
C      18   TRANS.TAB         TABLE OF INDIVIDUAL TRANSITIONS
C      19   SHELL.SUM         TABLE OF TRANSITIONS SUMMED OVER SHELLS
C
C     PLOTTAB FORMATTED OUTPUT
C     ==================================================================
C     PLOTTAB FORMATTED OUTPUT MAY BE DISPLAYED ON VIRTUALLY ANY
C     PLOTTING DEVICE - ON SCREEN OR HARDCOPY - SEE, UCRL-ID-110240
C     (MARCH 1992) - FOR A DESCRIPTION OF THE CODE - CONTACT THE
C     AUTHOR OF THIS CODE FOR A COPY OF PLOTTAB.
C
C     INPUT PARAMETERS
C     ==================================================================
C     LINE  COLS.  FORMAT  DESCRIPTION
C     ----  -----  ------  -----------
C       1    1- 5    I5    LISTING OPTION
C                          = 0 - NO OUTPUT LISTING
C                          = 1 - SUMMARY OF EACH SUBSHELL - NO SPECTRA
C                          = 2 - SUMMARY AND SPECTRA OF EACH SUBSHELL
C            6-10    I5    ENDL OUTPUT OPTION
C                          = 0 - NO ENDL OUTPUT
C                          = 1 - ENDL OUTPUT
C           11-15    I5    SPECTRA AND INTEGRAL OPTION
C                          = 0 - NO PLOTTAB OUTPUT
C                          = 1 - PLOTTAB OUTPUT
C           16-20    I5    FLUORESCENCE VS. Z (BY SUBSHELL) OPTION
C                          = 0 - NO PLOTTAB OUTPUT
C                          = 1 - PLOTTAB OUTPUT
C           21-25    I5    LISTING UNIT
C                          = 0 - TO FILE RELAX.LST
C                          = 1 - ON-LINE - DO NOT OPEN RELAX.LST
C           26-30    I5    RESULTS BY SHELL SUMMED OVER SUBSHELL
C                          = 0 - NO SHELL RESULTS
C                          = 1 - SHELL RESULTS
C     2-N    1- 5    I5    MINIMUM Z
C            6-10    I5    MAXIMUM Z
C           11-15    I5    MINIMUM SUBSHELL DESIGNATOR
C           16-20    I5    MAXIMUM SUBSHELL DESIGNATOR
C           21-25    I5    DESCRIPTION OF FINAL ATOM
C                          = 0 - RETURN TO NEUTRALITY
C                          = 1 - ISOLATED
C
C     LINE 2 MAY BE REPEATED ANY NUMBER OF TIMES. EACH ELEMENT/SUBSHELL
C     REQUEST WILL BE PROCESSED SEPARATELY AND PROCESSING WILL CONTINUE
C     UNTIL AN END OF FILE IS READ ON THE INPUT FILE.
C
C     THE MINIMUM AND MAXIMUM Z MUST BE BETWEEN 1 AND 100 AND THE
C     MAXIMUM MUST BE EQUAL TO OR GREATER THAN THE MINIMUM. SIMILARLY
C     THE MINIMUM AND MAXIMUM SUBSHELL DESIGNATORS MUST BE BETWEEN 1
C     AND 61 AND THE MAXIMUM MUST BE EQUAL TO OR GREATER THAN THE
C     MINIMUM. IF THE INPUT PARAMETERS DO NOT SATISFY THESE
C     CONDITIONS THE PROGRAMM WILL TERMINATE EXECUTION.
C
C     EXAMPLE INPUT NO. 1
C     -------------------
C     PROCESS THE ENTIRE EADL LIBRARY WITH ALL OUTPUT OPTIONS ON. THE
C     FOLLOWING 2 INPUT LINES ARE REQUIRED,
C
C         2    1    1    1    0
C         1  100    1   61    0
C
C     EXAMPLE INPUT NO. 2
C     -------------------
C     PROCESS THE MAGNESIUM (Z=12), K SHELL (SUBSHELL DESIGNATOR = 1)
C     AND ONLY OUTPUT A FULL LISTING. THE FOLLOWING 2 INPUT LINES ARE
C     REQUIRED,
C
C         2    0    0    0    0
C        12   12    1    1    0
C
C     EXAMPLE INPUT NO. 3
C     -------------------
C     PROCESS ALL SUBSHELLS OF Z = 6 THROUGH 10 AND 90 THROUGH 100.
C     OUTPUT A SUMMARY OF EACH SUBSHELL AND PLOTTAB FORMATTED SPECTRA.
C     THE FOLLOWING 3 INPUT LINES ARE REQUIRED,
C
C         1    0    1    0    0
C         6   10    1   61    0
C        90  100    1   61    0
C
C      DEFINITION OF ENDL SUBSHELL DESIGNATORS
C=======================================================================
C  1 = K (1s1/2)   17 = N23 (4p)    32 = O4 (5d3/2)  47 = P5 (6d5/2)
C  2 = L (2)       18 = N2 (4p1/2)  33 = O5 (5d5/2)  48 = P67 (6f)
C  3 = L1 (2s1/2)  19 = N3 (4p3/2)  34 = O67 (5f)    49 = P6 (6f5/2)
C  4 = L23 (2p)    20 = N45 (4d)    35 = O6 (5f5/2)  50 = P7 (6f7/2)
C  5 = L2 (2p1/2)  21 = N4 (4d3/2)  36 = O7 (5f7/2)  51 = P89 (6g)
C  6 = L3 (2p3/2)  22 = N5 (4d5/2)  37 = O89 (5g)    52 = P8 (6g7/2)
C  7 = M (3)       23 = N67 (4f)    38 = O8 (5g7/2)  53 = P9 (6g9/2)
C  8 = M1 (3s1/2)  24 = N6 (4f5/2)  39 = O9 (5g9/2)  54 = P1011 (6h)
C  9 = M23 (3p)    25 = N7 (4f7/2)  40 = P (6)       55 = P10 (6h9/2)
C 10 = M2 (3p1/2)  26 = O (5)       41 = P1 (6s1/2)  56 = P11 (6h11/2)
C 11 = M3 (3p3/2)  27 = O1 (5s1/2)  42 = P23 (6p)    57 = Q (7)
C 12 = M45 (3d)    28 = O23 (5p)    43 = P2 (6p1/2)  58 = Q1 (7s1/2)
C 13 = M4 (3d3/2)  29 = O2 (5p1/2)  44 = P3 (6p3/2)  59 = Q23 (7p)
C 14 = M5 (3d5/2)  30 = O3 (5p3/2)  45 = P45 (6d)    60 = Q2 (7p1/2)
C 15 = N (4)       31 = O45 (5d)    46 = P4 (6d3/2)  61 = Q3 (7p3/2)
C 16 = N1 (4s1/2)
C=======================================================================
C
C     DEFINITION OF VARIABLES
C     ==================================================================
C     BASIS DATA
C     ------------------------------------------------------------------
C     IZGET   = Z FOR ELEMENT (1 TO 100)
C     ISUBGT  = SUBSHELL INDEX (1 TO 61)
C     MAXSUB  = MAXIMUM SUBSHELL INDEX THAT MUST BE USED IN CALCULATION
C     MAXTRA  = MAXIMUM SUBSHELL INDEX THAT HAS TRANSITIONS TO IT
C     ATWTGT  = ATOMIC WEIGHT OF ELEMENT IZGET
C     EBMEV   = SUBSHELL BINDING ENERGY (MEV)
C     EBEV    = SUBSHELL BINDING ENERGY (EV)
C     TR      = RADIATIVE TRANSITION PROBABILITIES
C     TNR     = NON-RADIATIVE TRANSITION PROBABILITIES
C     NSUBR   = RADIATIVE TRANSITIONS READ FLAG
C     NSUBN   = NON-RADIATIVE TRANSITIONS READ FLAG
C     ------------------------------------------------------------------
C     SPECTRA FOR ONE SUBSHELL - CALCULATED BY THIS CODE
C     ------------------------------------------------------------------
C     ERO     = ENERGY OF RADIATIVE TRANSITION (EV)
C     PROBRO  = PROBABILITY OF RADIATIVE TRANSITION
C     NR      = NUMBER OF RADIATIVE TRANSITIONS
C     ENO     = ENERGY OF NON-RADIATIVE TRANSITION (EV)
C     PROBNO  = PROBABILITY OF NON-RADIATIVE TRANSITION
C     NN      = NUMBER OF NON-RADIATIVE TRANSITIONS
C     ELO     = ENERGY OF FREE-BOUND TRANSITION (EV)
C     PROBLO  = PROBABILITY FREE-BOUND TRANSITION
C     NL      = NUMBER OF FREE-BOUND TRANSITIONS
C     ------------------------------------------------------------------
C     AVERAGE VALUES FOR ALL SUBSHELLS - CALCULATED BY THIS CODE
C     ------------------------------------------------------------------
C     OUTR    = AVERAGE NUMBER OF X-RAYS EMITTED
C     OUTN    = AVERAGE NUMBER OF ELECTRONS EMITTED
C     OUTL    = AVERAGE NUMBER OF FREE-BOUND TRANSITIONS
C     OUTRE   = TOTAL EMITTED X-RAY ENERGY (EV)
C     OUTNE   = TOTAL EMITTED ELECTRON ENERGY (EV)
C     OUTLE   = TOTAL EMITTED FREE-BOUND ENERGY (EV)
C     OUTAVR  = AVERAGE EMITTED X-RAY ENERGY (EV)
C     OUTAVN  = AVERAGE EMITTED ELECTRON ENERGY (EV)
C     OUTAVL  = AVERAGE EMITTED FREE-BOUND ENERGY (EV)
C
C     USING THE RESULTS
C     ==================================================================
C     AFTER RESULTS HAVE BEEN CALCULATED FOR EACH SUBSHELL SUBROUTINE
C     USER1 IS CALLED - AT THIS POINT THE SPECTRA FOR THE SUBSHELL ARE
C     AVAILABLE FOR USE.
C
C     AFTER RESULTS HAVE BEEN CALCULATED FOR ALL SUBSHELLS OF AN
C     ELEMENT SUBROUTINE USER2 IS CALLED - AT THIS POINT THE AVERAGE
C     VALUES FOR ALL SUBSHELLS OF THE ELEMENT ARE AVAILABLE FOR USE.
C
C***** COMPUTER DEPENDENT CODING ******
C
C     2014/9/14 - ALL COMPUTER DEPENDENT CODE REMOVED.
C
C***** COMPUTER DEPENDENT CODING ******
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      REAL*8 MEV2EV
      CHARACTER*1 ZABCD,SHBCD
      CHARACTER*8 OPCODE
      INTEGER OUTP,OTAPE
      COMMON/UNITS/INP,OUTP,ITAPE,OTAPE,KTAPE,LTAPE,
     1 IPLOT1,IPLOT2,IPLOT3,IPLOT4
      COMMON/WHERE/ATWTGT,IZGET,ISUBGT,MAXSUB,MAXTRA
      COMMON/WHEREC/ZABCD(12),SHBCD(12)
      COMMON/SPECTR/ERO(5000),PROBRO(5000),NR
      COMMON/SPECTN/ENO(5000),PROBNO(5000),NN
      COMMON/SPECTL/ELO(39),PROBLO(39),NL
      COMMON/TRANS/TR(39,39),TNR(39,39,39),NSUBR(39),NSUBN(39)
      COMMON/EADL/EBMEV(39),EBEV(39),ELECT(39),HOLES(39)
      COMMON/OUTER/OUTR(39),OUTN(39),OUTL(39),OUTRE(39),OUTNE(39),
     1 OUTLE(39),OUTAVR(39),OUTAVN(39),OUTAVL(39)
      COMMON/FLUORS/FLUOR(100,39,6),MSHELL,MSUB1,MSUB2
      COMMON/OUTOPS/IMOUT(6)
      DIMENSION OPCODE(5),TNORM(39),SHELLP(7,7),SHELLE(7,7,7)
      DATA IZNEXT/0/
      DATA IZNOW/0/
      DATA OPCODE/
     1 'None    ',
     2 'Summary ',
     3 'Full    ',
     4 'No      ',
     5 'Yes     '/
      DATA ZERO/0.0D+00/
      DATA ONE/1.0D+00/
      DATA MEV2EV/1.0D+06/
C-----DEFINE I/O UNITS.
      INP=5
      OUTP=6
      ITAPE=10
      OTAPE=11
      KTAPE=18
      LTAPE=19
      IPLOT1=12
      IPLOT2=14
      IPLOT3=15
      IPLOT4=17
      OPEN(INP,FILE='RELAX.INP')
      OPEN(ITAPE,FILE='ENDL.IN')
      OPEN(OTAPE,FILE='ENDL.OUT')
      OPEN(KTAPE,FILE='TRANS.TAB')
      OPEN(LTAPE,FILE='SHELL.SUM')
      OPEN(IPLOT1,FILE='PLOTTAB.CUR')
      OPEN(IPLOT2,FILE='PLOTTAB.PNT')
      OPEN(IPLOT3,FILE='PLOTTAB.INP')
      OPEN(IPLOT4,FILE='PLOTTAB.CUR.SHELL')
C
C     INITIALIZE FLUORESCENCE YIELD.
C
      DO 10 I=1,100
      DO 10 J=1,39
      DO 10 K=1,6
   10 FLUOR(I,J,K)=ZERO
C
C     READ AND LIST OUTPUT OPTIONS.
C
      READ(INP,420,END=330,ERR=330) IMOUT
      IF(IMOUT(1).LT.0) IMOUT(1)=0
      IF(IMOUT(1).GT.2) IMOUT(1)=2
      IF(IMOUT(2).NE.0) IMOUT(2)=1
      IF(IMOUT(3).NE.0) IMOUT(3)=1
      IF(IMOUT(4).NE.0) IMOUT(4)=1
      IF(IMOUT(5).NE.0) IMOUT(5)=1
      IF(IMOUT(6).NE.0) IMOUT(6)=1
      I1=IMOUT(1)+1
      I2=IMOUT(2)+4
      I3=IMOUT(3)+4
      I4=IMOUT(4)+4
      I6=IMOUT(6)+4
C-----IF REQUESTED OPEN FILE FOR OUTPUT REPORT.
      IF(IMOUT(5).EQ.0) OPEN(OUTP,FILE='RELAX.LST')
C-----IDENTIFY PROGRAM.
      WRITE(OUTP,430)
      IF(IMOUT(5).eq.0)
     1 WRITE(*   ,430)
C-----LIST DESCRIPTION OF OPTIONS.
      WRITE(OUTP,450) OPCODE(I1),OPCODE(I2),OPCODE(I3),OPCODE(I4)
      IF(IMOUT(5).eq.0)
     1 WRITE(*   ,450) OPCODE(I1),OPCODE(I2),OPCODE(I3),OPCODE(I4)
      IF(IMOUT(5).EQ.0) WRITE(OUTP,460)
      IF(IMOUT(5).NE.0) WRITE(OUTP,470)
      WRITE(OUTP,480) OPCODE(I6)
      IF(IMOUT(5).eq.0)
     1 WRITE(*   ,480) OPCODE(I6)
C
C     READ AND LIST NEXT REQUESTED Z AND SUBSHELL INDEX.
C
   20 READ(INP,420,END=330,ERR=330) IZ1,IZ2,ISUB1,ISUB2,MYWAY
      WRITE(OUTP,490) IZ1,IZ2,ISUB1,ISUB2
      IF(MYWAY.EQ.0) WRITE(OUTP,500)
      IF(MYWAY.NE.0) WRITE(OUTP,510)
      IF(IMOUT(5).eq.0) then
      WRITE(*   ,490) IZ1,IZ2,ISUB1,ISUB2
      IF(MYWAY.EQ.0) WRITE(*   ,500)
      IF(MYWAY.NE.0) WRITE(*   ,510)
      endif
C-----STOP ON ILLEGAL DATA.
      IF(IZ1.LT.1.OR.IZ1.GT.100) GO TO 320
      IF(IZ2.LT.1.OR.IZ2.GT.100) GO TO 320
      IF(IZ1.GT.IZ2) GO TO 320
      IF(ISUB1.LT.1.OR.ISUB2.LT.1) GO TO 320
      IF(ISUB1.GT.61.OR.ISUB2.GT.61) GO TO 320
      IF(ISUB1.GT.ISUB2) GO TO 320
C
C     LOOP OVER ELEMENTS.
C
      DO 310 IZGET=IZ1,IZ2
C-----DEFINE ELEMENT.
      CALL ZAHOL(1000*IZGET,ZABCD)
C-----INITIALIZE DATA FOUND FLAG.
      IGOTIT=0
C-----IF DATA IS ALREADY IN CORE USE IT.
      IF(IZGET.EQ.IZNOW) GO TO 250
C-----INITIALIZE BINDING ENERGIES, ELECTRONS PER SUBSHELL, SUBSHELL
C-----USED FLAGS AND AVERAGE VALUES.
      DO 30 I=1,39
      EBMEV(I)=ZERO
      ELECT(I)=ZERO
      TNORM(I)=ZERO
      NSUBR(I)=0
      NSUBN(I)=0
      OUTR(I)=ZERO
      OUTN(I)=ZERO
      OUTL(I)=ZERO
      OUTRE(I)=ZERO
      OUTNE(I)=ZERO
      OUTLE(I)=ZERO
      OUTAVR(I)=ZERO
      OUTAVN(I)=ZERO
   30 OUTAVL(I)=ZERO
      MAXSUB=1
      MAXTRA=0
C-----IF REQUESTED Z IS THE NEXT ONE THE FIRST LINE IS ALREADY
C-----IN CORE.
      IF(IZGET.EQ.IZNEXT) GO TO 50
C-----IF PAST REQUESTED Z REWIND BEFORE SEARCH.
      IF(IZGET.LT.IZNEXT) REWIND ITAPE
      IZNEXT=0
      IZNOW=0
C
C     LOCATE REQUESTED Z.
C
C-----READ ALL RADIATIVE AND NON-RADIATIVE TRANSITIONS.
   40 READ(ITAPE,340,END=160,ERR=160) IZ,ATWT
C-----CHECK FOR REQUESTED Z.
      IF(IZ.lt.IZGET) go to 60
      IF(IZ.gt.IZGET) go to 170
C-----FIND RELAXATION DATA.
   50 READ(ITAPE,350,END=160,ERR=160) IC,II,SUB1
C-----CHECK FOR BINDING ENERGIES.
      IF(IC.EQ.91.AND.II.EQ.913) GO TO 70
C-----CHECK FOR ELECTRONS PER SUBSHELL.
      IF(IC.EQ.91.AND.II.EQ.912) GO TO 90
C-----CHECK FOR RADIATIVE TRANSITIONS.
      IF(IC.EQ.92.AND.II.EQ.931) GO TO 100
C-----CHECK FOR NON-RADIATIVE TRANSITIONS.
      IF(IC.EQ.92.AND.II.EQ.932) GO TO 130
C-----SKIP SECTION.
   60 READ(ITAPE,360,END=160,ERR=160) IFLAG
      IF(IFLAG.gt.0) go to 40
      go to 60
C
C     READ BINDING ENERGIES.
C
C-----SAVE ATOMIC WEIGHT FOR OUTPUT.
   70 ATWTGT=ATWT
C-----INDICATE DATA HAS BEEN FOUND.
      IGOTIT=1
   80 READ(ITAPE,380,END=160,ERR=160) SUB1,END1,IFLAG
      IF(IFLAG.gt.0) go to 40
      KSUBX1=SUB1
      KSUB1=ISDOWN(KSUBX1)
      EBMEV(KSUB1)=END1
      EBEV(KSUB1)=MEV2EV*EBMEV(KSUB1)
      IF(KSUB1.GT.MAXSUB) MAXSUB=KSUB1
      GO TO 80
C
C     READ ELECTRONS PER SUBSHELL.
C
   90 READ(ITAPE,380,END=160,ERR=160) SUB1,END1,IFLAG
      IF(IFLAG.gt.0) go to 40
      KSUBX1=SUB1
      KSUB1=ISDOWN(KSUBX1)
      ELECT(KSUB1)=END1
      GO TO 90
C
C     READ RADIATIVE TRANSITIONS.
C
C-----INDICATE SUBSHELL USED AND INITIALIZE ARRAYS.
  100 KSUBX1=SUB1
      KSUB1=ISDOWN(KSUBX1)
      NSUBR(KSUB1)=1
      IF(KSUB1.GT.MAXTRA) MAXTRA=KSUB1
      DO 110 J=1,39
  110 TR(KSUB1,J)=ZERO
  120 READ(ITAPE,370,END=160,ERR=160) SUB2,PROB1,IFLAG
      IF(IFLAG.gt.0) go to 40
      KSUBX2=SUB2
      KSUB2=ISDOWN(KSUBX2)
      TR(KSUB1,KSUB2)=PROB1
      TNORM(KSUB1)=TNORM(KSUB1)+PROB1
      GO TO 120
C
C     READ NON-RADIATIVE TRANSITIONS.
C
C-----INDICATE SUBSHELL USED AND INITIALIZE ARRAYS.
  130 KSUBX1=SUB1
      KSUB1=ISDOWN(KSUBX1)
      NSUBN(KSUB1)=1
      IF(KSUB1.GT.MAXTRA) MAXTRA=KSUB1
      DO 140 J=1,39
      DO 140 K=1,39
  140 TNR(KSUB1,J,K)=ZERO
  150 READ(ITAPE,390,END=160,ERR=160) SUB2,SUB3,PROB1,IFLAG
      IF(IFLAG.gt.0) go to 40
      KSUBX2=SUB2
      KSUBX3=SUB3
      KSUB2=ISDOWN(KSUBX2)
      KSUB3=ISDOWN(KSUBX3)
      TNR(KSUB1,KSUB2,KSUB3)=PROB1
      TNORM(KSUB1)=TNORM(KSUB1)+PROB1
      GO TO 150
C
C     ALL DATA READ. PROCESS IT.
C
C-----END OF ENDL DATA.
  160 IZ=200
  170 IZNEXT=IZ
      IZNOW=IZGET
C
C     THERE IS NOTHING TO DO IF NO DATA WAS FOUND.
C
      IF(IGOTIT.LE.0) GO TO 310
C
C     RE-NORMALIZE TRANSITIONS. ONLY USE DATA IF NON-RADIATIVE DATA
C     ARE GIVEN.
C
      DO 210 ISUB=1,MAXSUB
      IF(NSUBN(ISUB).GT.0.AND.TNORM(ISUB).GT.ZERO) GO TO 180
      NSUBN(ISUB)=0
      NSUBR(ISUB)=0
      GO TO 210
  180 RNORMR=ONE/TNORM(ISUB)
      RNORMN=ONE/TNORM(ISUB)
      DO 200 I=ISUB,MAXSUB
      TR(ISUB,I)=TR(ISUB,I)*RNORMR
      DO 190 J=ISUB,MAXSUB
  190 TNR(ISUB,I,J)=TNR(ISUB,I,J)*RNORMN
  200 CONTINUE
  210 CONTINUE
C
C     DEFINE AND OUTPUT SHELL RESULTS.
C
C-----INITIALIZE.
      DO 220 I=1,7
      DO 220 J=1,7
      SHELLP(I,J)=0.0
      DO 220 K=1,7
  220 SHELLE(I,J,K)=0.0
C-----SUM OVER SUBSHELLS IN EACH SHELL.
      DO 240 I=1,MAXSUB
      IF(NSUBN(I).LE.0.OR.TNORM(I).LE.ZERO) GO TO 240
      IS=ISUP(I)
      CALL SHELL1(IS,ISH)
      DO 230 J=I,MAXSUB
      JS=ISUP(J)
      CALL SHELL1(JS,JSH)
      SHELLP(ISH,JSH)=SHELLP(ISH,JSH)+TR(I,J)
      DO 230 K=I,MAXSUB
      KS=ISUP(K)
      CALL SHELL1(KS,KSH)
  230 SHELLE(ISH,JSH,KSH)=SHELLE(ISH,JSH,KSH)+TNR(I,J,K)
  240 CONTINUE
C-----OUTPUT RESULTS.
      WRITE(LTAPE,410) IZGET
      WRITE(LTAPE,400) SHELLP
      WRITE(LTAPE,400) SHELLE
C
C     LOOP OVER SUBSHELLS.
C
  250 DO 300 KSUBGT=ISUB1,ISUB2
      ISUBGT=ISDOWN(KSUBGT)
C-----IGNORE UNUSED SUBSHELLS.
      IF(ISUBGT.LE.0) GO TO 300
C-----IGNORE SUBSHELLS WITH NO BINDING ENERGY.
      IF(EBMEV(ISUBGT).LE.ZERO) GO TO 300
C-----DEFINE SUBSHELL.
      CALL SUBHOL(KSUBGT,SHBCD)
C-----INITIALIZE NUMBER OF EMISSIONS.
      NR=0
      NN=0
      NL=0
C-----CALCULATION NOT REQUIRED IF NO TRANSITION DATA.
      IF(NSUBN(ISUBGT).LE.0) GO TO 260
      IF(ISUBGT.LE.MAXTRA) GO TO 270
C-----NO TRANSITIONS. ASSIGN EVERYTHING TO LOCAL FOR OUTPUT.
  260 OUTL(ISUBGT)=ONE
      OUTLE(ISUBGT)=EBEV(ISUBGT)
      OUTAVL(ISUBGT)=EBEV(ISUBGT)
      GO TO 290
C-----DEFINE SHELL INDICES.
  270 CALL MYSHEL(KSUBGT,MSUB1,MSUB2)
      MSUB1=ISDOWN(MSUB1)
      MSUB2=ISDOWN(MSUB2)
C-----CALCULATE SPECTRA AND AVERAGE VALUES.
      IF(MYWAY.NE.0) GO TO 280
      CALL SPECTA
      GO TO 290
  280 CALL SPECTB
C-----LIST RESULTS.
  290 CALL LIST1
C
C     END OF SUBSHELL. USER INTERFACE ROUTINE.
C
      CALL USER1
  300 CONTINUE
C
C     END OF ELEMENT. USER INTERFACE ROUTINE.
C
      CALL USER2
C
C     END OF ELEMENT LOOP
C
  310 CONTINUE
      GO TO 20
C
C     ERROR IN INPUT.
C
  320 WRITE(OUTP,520)
C
C     END OF RUN.
C
C     LIST FLUORESCENCE YIELD.
C
  330 CALL LIST2
      WRITE(OUTP,440)
      STOP
  340 FORMAT(I3,9X,E12.4)
  350 FORMAT(I2,I3,15X,E12.4)
  360 FORMAT(71X,I1)
  370 FORMAT(2E16.9,39X,I1)
  380 FORMAT(E16.9,E16.9,39X,I1)
  390 FORMAT(3E16.9,23X,I1)
  400 FORMAT(1PE11.4,1PE11.4,1PE11.4,1PE11.4,1PE11.4,1PE11.4,1PE11.4)
  410 FORMAT(I11)
  420 FORMAT(14I5)
  430 FORMAT(///' Calculation of X-Ray and Electron Relaxation',
     1 ' Spectra (RELAX 2014)')
  440 FORMAT(1X,72('=')/
     1 ' Bound-Bound X-Ray and Electron Spectra are based on EADL',
     2 ' Relaxation data.'//
     3 ' Free-Bound X-Ray Spectrum assumes the atom will return to',
     4 ' Neutrality by'/
     5 ' electron transitions directly from the continuum to fill all',
     6 ' remaining'/
     7 ' electron holes, resulting in the emission of X-Rays with',
     8 ' energy equal'/
     9 ' to the binding energy of the electron holes.')
  450 FORMAT(1X,72('=')/
     1 ' Output Listing............................',A8/
     2 ' ENDL Output...............................',A8/
     3 ' Spectra and Integral......................',A8/
     4 ' Fluorescence Yield vs. Z by Subshell......',A8)
  460 FORMAT(
     1 ' Listing Unit..............................RELAX.LST')
  470 FORMAT(
     1 ' Listing Unit..............................ON-LINE')
  480 FORMAT(
     1 ' Shell Results Summed Over Subshells.......',A8)
  490 FORMAT(1X,72('=')/
     1 ' Process Z.................................',I4,' through',I4/
     2 '         Subshell..........................',I4,' through',I4)
  500 FORMAT(
     1 ' Description of Final Atom................. Neutral')
  510 FORMAT(
     1 ' Description of Final Atom................. Isolated')
  520 FORMAT(1X,72('=')/
     1 ' Error in input ranges.'/
     2 ' Z range may be 1 to 100.'/
     3 ' Subshell range may be 1 to 61.'/
     4 ' Execution Terminated.')
      END
      SUBROUTINE MYSHEL(ISUB,ISUB1,ISUB2)
C=======================================================================
C
C     DEFINE SUBSHELL INDICES FOR SAME SHELL.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      DIMENSION MYSUB(2,7)
C-----SUBSHELL DESIGNATOR RANGES FOR EACH SHELL.
      DATA MYSUB/
     1    1,    1,
     2    3,    6,
     3    8,   14,
     4   16,   25,
     5   27,   39,
     6   41,   56,
     7   58,   61/
      DO 10 ISHELL=1,7
      IF(ISUB.GE.MYSUB(1,ISHELL).AND.
     1   ISUB.LE.MYSUB(2,ISHELL)) GO TO 20
   10 CONTINUE
      ISHELL=7
   20 ISUB1=MYSUB(1,ISHELL)
      ISUB2=MYSUB(2,ISHELL)
      RETURN
      END
      SUBROUTINE SHELL1(ISUB,ISHELL)
C=======================================================================
C
C     DEFINE SHELL CORRESPONDING TO SUBSHELL.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      DIMENSION MYSUB(2,7)
C-----SUBSHELL DESIGNATOR RANGES FOR EACH SHELL.
      DATA MYSUB/
     1    1,    1,
     2    3,    6,
     3    8,   14,
     4   16,   25,
     5   27,   39,
     6   41,   56,
     7   58,   61/
      DO 10 ISHELL=1,7
      IF(ISUB.GE.MYSUB(1,ISHELL).AND.
     1   ISUB.LE.MYSUB(2,ISHELL)) GO TO 20
   10 CONTINUE
      ISHELL=7
   20 RETURN
      END
      SUBROUTINE SHELL2(ISUB,ISHELL)
C=======================================================================
C
C     DEFINE SHELL CORRESPONDING TO SUBSHELL+L1,L2,L3
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      DIMENSION MYSUB(2,9)
C-----SUBSHELL DESIGNATOR RANGES FOR EACH SHELL.
      DATA MYSUB/
     1    1,    1,
     2    3,    3,
     3    5,    5,
     4    6,    6,
     5    8,   14,
     6   16,   25,
     7   27,   39,
     8   41,   56,
     9   58,   61/
      DO 10 ISHELL=1,9
      IF(ISUB.GE.MYSUB(1,ISHELL).AND.
     1   ISUB.LE.MYSUB(2,ISHELL)) GO TO 20
   10 CONTINUE
      ISHELL=9
   20 RETURN
      END
      SUBROUTINE LIST1
C=======================================================================
C
C     LIST SUBSHELL RESULTS
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*1 ZABCD,SHBCD,SHBCD1,SHBCD2,SHBCD3,STARX,STAR1,BLANK,
     1 CROSSX,CROSS1
      INTEGER OUTP,OTAPE
      COMMON/UNITS/INP,OUTP,ITAPE,OTAPE,KTAPE,LTAPE,
     1 IPLOT1,IPLOT2,IPLOT3,IPLOT4
      COMMON/WHERE/ATWTGT,IZGET,ISUBGT,MAXSUB,MAXTRA
      COMMON/WHEREC/ZABCD(12),SHBCD(12)
      COMMON/SPECTR/ERO(5000),PROBRO(5000),NR
      COMMON/SPECTN/ENO(5000),PROBNO(5000),NN
      COMMON/SPECTL/ELO(39),PROBLO(39),NL
      COMMON/EADL/EBMEV(39),EBEV(39),ELECT(39),HOLES(39)
      COMMON/OUTER/OUTR(39),OUTN(39),OUTL(39),OUTRE(39),OUTNE(39),
     1 OUTLE(39),OUTAVR(39),OUTAVN(39),OUTAVL(39)
      COMMON/INDEXS/INDEXR(5000),INDEXN(5000),INDEXL(39)
      COMMON/OUTOPS/IMOUT(6)
      DIMENSION SHBCD1(12),SHBCD2(12),SHBCD3(12)
      DATA STARX/'*'/
      DATA CROSSX/'+'/
      DATA BLANK/' '/
      DATA IZLAST/0/
      DATA ZERO/0.0D+00/
      DATA ONE/1.0D+00/
      DATA HOLMIN/1.0D-10/
C
C     NOTHING TO DO IF LISTED OUTPUT NOT REQUESTED.
C
      IF(IMOUT(1).LE.0) GO TO 40
C-----NOTHING TO DO IF SUBSHELL ENERGY IS NOT DEFINED.
      IF(EBMEV(ISUBGT).LE.ZERO) RETURN
C
C     ONLY A SUMMARY OF REMAINING SUBSHELLS AFTER ALL SUBSHELLS WITH
C     TRANSITIONS HAVE BEEN PROCESSED.
C
      IF(ISUBGT.LE.MAXTRA) GO TO 10
      IF(IZGET.NE.IZLAST) WRITE(OUTP,280) ZABCD
      IZLAST=IZGET
      II=ISUP(ISUBGT)
      CALL SUB4(II,SHBCD1)
      WRITE(OUTP,240) (SHBCD1(K),K=1,4),EBEV(ISUBGT),ELECT(ISUBGT)
      RETURN
C-----FRACTION OF ENERGY TO RADIATIVE/NON-RADIATIVE/LOCAL/TOTAL.
   10 FRACTR=100.0*OUTRE(ISUBGT)/EBEV(ISUBGT)
      FRACTN=100.0*OUTNE(ISUBGT)/EBEV(ISUBGT)
      FRACTL=100.0*OUTLE(ISUBGT)/EBEV(ISUBGT)
      FRACT=100.0
C
C     PRINT SUBSHELL SUMMARY.
C
      WRITE(OUTP,220) ZABCD,SHBCD,OUTRE(ISUBGT),FRACTR,
     1 OUTNE(ISUBGT),FRACTN,OUTLE(ISUBGT),FRACTL,EBEV(ISUBGT),
     2 FRACT,OUTAVR(ISUBGT),OUTAVN(ISUBGT),OUTAVL(ISUBGT),
     3 OUTR(ISUBGT),OUTN(ISUBGT),OUTL(ISUBGT)
C
C     INITIAL AND FINAL ELECTRONS AND HOLES.
C
      WRITE(OUTP,230)
      SUM1=ZERO
      SUM2=ZERO
      SUM3=ZERO
      SUM4=ZERO
      HOLEX=ONE
      IF(ELECT(ISUBGT).LT.ONE) HOLEX=ELECT(ISUBGT)
      IMNEG=0
      ILIST=0
      DO 30 I=ISUBGT,MAXSUB
      IF(EBMEV(I).LE.ZERO) GO TO 30
      II=ISUP(I)
      CALL SUB4(II,SHBCD1)
      HOLE0=ZERO
      IF(I.EQ.ISUBGT) HOLE0=HOLEX
      ELECT0=ELECT(I)-HOLE0
      HOLE1=HOLES(I)
      IF(DABS(HOLEI).LE.HOLMIN) HOLEI=ZERO
      ELECT1=ELECT(I)-HOLE1
      CROSS1=BLANK
      IF(I.EQ.ISUBGT.AND.HOLEX.LT.ONE) CROSS1=CROSSX
      STAR1=BLANK
      IF(DABS(ELECT1).LE.HOLMIN) ELECT1=ZERO
      IF(ELECT1.GE.ZERO) GO TO 20
      STAR1=STARX
      IMNEG=1
   20 WRITE(OUTP,240) (SHBCD1(K),K=1,4),EBEV(I),ELECT0,HOLE0,CROSS1,
     1 ELECT1,STAR1,HOLE1
      ILIST=ILIST+1
      SUM1=SUM1+ELECT0
      SUM2=SUM2+HOLE0
      SUM3=SUM3+ELECT1
      SUM4=SUM4+HOLES(I)
   30 CONTINUE
      IF(ILIST.GT.1) WRITE(OUTP,250) SUM1,SUM2,SUM3,SUM4
      IF(IMNEG.NE.0) WRITE(OUTP,260)
      IF(HOLEX.LT.ONE) WRITE(OUTP,270) HOLEX
C
C     OUTPUT SHELL RESULTS SUMMED OVER SUBSHELLS.
C
   40 CALL SHELLO
C
C     NOTHING ELSE TO DO IF LISTED SPECTRA NOT REQUESTED.
C
      IF(IMOUT(1).LE.1) RETURN
C
C     LIST SPECTRA.
C
C-----X-RAYS
      IF(NR.GT.0) GO TO 50
      WRITE(OUTP,170)
      GO TO 70
   50 WRITE(OUTP,140)
      SUMFR=ZERO
      SUMHOL=ZERO
      DO 60 I=1,NR
      ERO(I)=ERO(I)
      FRACT=100.0*ERO(I)*PROBRO(I)/EBEV(ISUBGT)
      SUMFR=SUMFR+FRACT
      SUMHOL=SUMHOL+PROBRO(I)
      II1X=INDEXR(I)/100
      II2X=MOD(INDEXR(I),100)
      II1=ISUP(II1X)
      II2=ISUP(II2X)
      CALL SUB4(II1,SHBCD1)
      CALL SUB4(II2,SHBCD2)
   60 WRITE(OUTP,200) I,ERO(I),PROBRO(I),FRACT,(SHBCD1(K),K=1,4),
     1 (SHBCD2(K),K=1,4)
      IF(NR.GT.1) WRITE(OUTP,210) SUMHOL,SUMFR
C-----ELECTRONS
   70 IF(NN.GT.0) GO TO 80
      WRITE(OUTP,180)
      GO TO 100
   80 WRITE(OUTP,150)
      SUMFN=ZERO
      SUMHOL=ZERO
      DO 90 I=1,NN
      ENO(I)=ENO(I)
      FRACT=100.0*ENO(I)*PROBNO(I)/EBEV(ISUBGT)
      SUMFN=SUMFN+FRACT
      SUMHOL=SUMHOL+PROBNO(I)
      II1X=INDEXN(I)/10000
      II2X=MOD(INDEXN(I),10000)/100
      II3X=MOD(INDEXN(I),100)
      II1=ISUP(II1X)
      II2=ISUP(II2X)
      II3=ISUP(II3X)
      CALL SUB4(II1,SHBCD1)
      CALL SUB4(II2,SHBCD2)
      CALL SUB4(II3,SHBCD3)
   90 WRITE(OUTP,200) I,ENO(I),PROBNO(I),FRACT,(SHBCD1(K),K=1,4),
     1 (SHBCD2(K),K=1,4),(SHBCD3(K),K=1,4)
      IF(NN.GT.1) WRITE(OUTP,210) SUMHOL,SUMFN
C-----FREE-BOUND
  100 IF(NL.GT.0) GO TO 110
      WRITE(OUTP,190)
      GO TO 130
  110 WRITE(OUTP,160)
      SUMFL=ZERO
      SUMHOL=ZERO
      DO 120 I=1,NL
      ELO(I)=ELO(I)
      FRACT=100.0*ELO(I)*PROBLO(I)/EBEV(ISUBGT)
      SUMFL=SUMFL+FRACT
      SUMHOL=SUMHOL+PROBLO(I)
      II1=ISUP(INDEXL(I))
      CALL SUB4(II1,SHBCD1)
      WRITE(OUTP,200) I,ELO(I),PROBLO(I),FRACT,(SHBCD1(K),K=1,4)
  120 CONTINUE
      IF(NL.GT.1) WRITE(OUTP,210) SUMHOL,SUMFL
  130 RETURN
  140 FORMAT(1X,72('=')/' Bound-Bound X-Ray Spectrum by Subshell'/
     1       1X,72('=')/
     2 ' Index   Emitted      Probability per    Per-cent of',
     3 '     Transition'/
     4 '         Energy (eV)  Initial Vacancy    Binding Energy'/
     5 1X,72('='))
  150 FORMAT(1X,72('=')/' Bound-Bound Electron Spectrum'/1X,72('=')/
     1 ' Index   Emitted      Probability per    Per-cent of',
     2 '     Transition'/
     2 '         Energy (eV)  Initial Vacancy    Binding Energy'/
     3 1X,72('='))
  160 FORMAT(1X,72('=')/' Free-Bound X-Ray Spectrum'/1x,72('=')/
     1 ' Index   Emitted      Probability per    Per-cent of',
     2 '     Transition'/
     2 '         Energy (eV)  Initial Vacancy    Binding Energy'/
     3 1X,72('='))
  170 FORMAT(1X,72('=')/' No Bound-Bound X-Rays Emitted'/1X,72('='))
  180 FORMAT(1X,72('=')/' No Bound-Bound Electrons Emitted'/1X,72('='))
  190 FORMAT(1X,72('=')/' No Free-Bound X-Rays Emitted'/1X,72('='))
  200 FORMAT(I6,2X,F11.2,2X,F11.8,6X,F11.4,8X,4A1,4A1,4A1)
  210 FORMAT(16X,'Sum',2X,F11.8,6X,F11.4)
  220 FORMAT(1X,72('=')/1X,12A1,1X,12A1,
     1 ' Ensemble Averaged Values per Initial Vacancy'/1X,72('=')/
     2 ' Total Bound-Bound X-Ray Energy..........',F11.2,' eV',F11.4,
     3 ' %'/
     4 ' Total Bound-Bound Electron Energy.......',F11.2,' eV',F11.4,
     5 ' %'/
     6 ' Total Free-Bound X-Ray Energy (Local)...',F11.2,' eV',F11.4,
     7 ' %'/
     5 ' Binding Energy (Sum of Above Terms).....',F11.2,' eV',F11.4,
     9 ' %'/
     6 1X,72('-')/
     6 ' Average Bound-Bound X-Ray Energy........',F11.2,' eV/X-Ray'/
     7 ' Average Bound-Bound Electron Energy.....',F11.2,' eV/Electron'/
     8 ' Average Free-Bound X-Ray Energy.........',F11.2,' eV/X-Ray'/
     6 1X,72('-')/
     9 ' Average Number of Bound-Bound X-Rays....',F11.6/
     A ' Average Number of Bound-Bound Electrons.',F11.6/
     B ' Average Number of Free-Bound X-Rays.....',F11.6)
  230 FORMAT(1X,72('=')/' Distributions - Initial and Final',
     1 ' after All Bound-Bound Transitions'/1X,72('=')/
     2 ' Subshell  Binding               Initial      ',
     3 '            Final'/
     4 '           Energy (eV)   Electrons      Holes',
     5 '      Electrons      Holes'/1x,72('='))
  240 FORMAT(1X,4A1,4X,F11.2,3X,F11.6,F11.6,A1,3X,F11.6,A1,F10.6)
  250 FORMAT(17X,'Sum',3X,F11.6,F11.6,4X,F11.6,F11.6)
  260 FORMAT(1X,72('=')/
     1 ' * - The Final Electrons can be negative since the effect of'/
     2 '     Free-Bound transitions filling holes is not included here')
  270 FORMAT(1X,72('=')/
     1 ' + - Subshell initially contained only ',F8.6,' electrons.'/
     2 '     The above table is normalized to ',F8.6,' initial holes.'/
     3 '     All other results are per initial vacancy.')
  280 FORMAT(1X,72('=')/12A1,
     1 ' No Transitions for the following Subshells'/1X,72('=')/
     2 ' Subshell  Binding       Electrons'/
     4 '           Energy (eV)'/1X,72('='))
      END
      SUBROUTINE SHELLO
C=======================================================================
C
C     OUTPUT SHELL RESULTS SUMMED OVER SUBSHELLS.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*1 ZABCD,SHBCD,SHBCD1,SHBCD2,SHBCD3
      INTEGER OUTP,OTAPE
      COMMON/UNITS/INP,OUTP,ITAPE,OTAPE,KTAPE,LTAPE,
     1 IPLOT1,IPLOT2,IPLOT3,IPLOT4
      COMMON/WHERE/ATWTGT,IZGET,ISUBGT,MAXSUB,MAXTRA
      COMMON/WHEREC/ZABCD(12),SHBCD(12)
      COMMON/EADL/EBMEV(39),EBEV(39),ELECT(39),HOLES(39)
      COMMON/SPECTR/ERO(5000),PROBRO(5000),NR
      COMMON/INDEXS/INDEXR(5000),INDEXN(5000),INDEXL(39)
      COMMON/OUTOPS/IMOUT(6)
      DIMENSION SHBCD1(12),SHBCD2(12),SHBCD3(12),PSHELL(9,9),
     1 ESHELL(9,9),KSHELL(9,9),NSHELL(9)
C-----INDICES CORRESPONDING TO SUBSHELL INDICE ASSIGNED TO EACH SHELL.
      DATA NSHELL/1,3,5,6,7,15,26,40,57/
      DATA ZERO/0.0D+00/
C-----NOTHING TO DO IF OUTPUT NOT REQUESTED OR NO TRANSITIONS.
      IF(IMOUT(1).LE.0.AND.IMOUT(6).LE.0.OR.NR.LE.0) RETURN
      KSUB=ISUP(ISUBGT)
      CALL SUB4(KSUB,SHBCD3)
C-----INITIALIZE ARRAYS.
      DO 10 IS=1,9
      DO 10 KS=1,9
      KSHELL(IS,KS)=0
      PSHELL(IS,KS)=0.0
   10 ESHELL(IS,KS)=0.0
C-----SUM OVER INDIVIDUAL TRANSITIONS.
      DO 20 I=1,NR
      II1X=INDEXR(I)/100
      II2X=MOD(INDEXR(I),100)
      II1=ISUP(II1X)
      II2=ISUP(II2X)
      CALL SHELL2(II1,ISHEL1)
      CALL SHELL2(II2,ISHEL2)
      KSHELL(ISHEL1,ISHEL2)=KSHELL(ISHEL1,ISHEL2)+1
      PSHELL(ISHEL1,ISHEL2)=PSHELL(ISHEL1,ISHEL2)+PROBRO(I)
      ESHELL(ISHEL1,ISHEL2)=ESHELL(ISHEL1,ISHEL2)+
     1 ERO(I)*PROBRO(I)
   20 CONTINUE
C-----OUTPUT RESULTS SHELL RESULTS
      IF(IMOUT(1).GT.0) WRITE(OUTP,80)
      SUMFR=ZERO
      SUMHOL=ZERO
      DO 40 IS=1,9
      CALL SUB4(NSHELL(IS),SHBCD1)
      DO 30 KS=1,9
      IF(PSHELL(IS,KS).LE.0.0) GO TO 30
      FRACT=100.0*ESHELL(IS,KS)/EBEV(ISUBGT)
      SUMFR=SUMFR+FRACT
      SUMHOL=SUMHOL+PSHELL(IS,KS)
      CALL SUB4(NSHELL(KS),SHBCD2)
      AVERE=ESHELL(IS,KS)/PSHELL(IS,KS)
      IF(IMOUT(1).GT.0)
     1 WRITE(OUTP,100) AVERE,PSHELL(IS,KS),FRACT,(SHBCD1(K),K=1,4),
     2 (SHBCD2(K),K=1,4)
      IF(IMOUT(6).GT.0)
     1 WRITE(KTAPE,70) ZABCD,(SHBCD3(K),K=1,2),SHBCD1(1),SHBCD1(2),
     2 SHBCD2(1),SHBCD2(1),PSHELL(IS,KS),AVERE,EBEV(ISUBGT),FRACT,
     3 KSHELL(IS,KS)
   30 CONTINUE
   40 CONTINUE
      IF(IMOUT(1).GT.0) WRITE(OUTP,110) SUMHOL,SUMFR
C-----OUTPUT SUM OVER ALL TRANSITIONS.
      IF(IMOUT(1).GT.0) WRITE(OUTP,90)
      SUMFR=ZERO
      SUMHOL=ZERO
      DO 60 IS=1,9
      CALL SUB4(NSHELL(IS),SHBCD1)
      ESUM=0.0
      PSUM=0.0
      DO 50 KS=1,9
      IF(PSHELL(IS,KS).LE.0.0) GO TO 50
      ESUM=ESUM+ESHELL(IS,KS)
      PSUM=PSUM+PSHELL(IS,KS)
   50 CONTINUE
      IF(PSUM.LE.0.0) GO TO 60
      FRACT=100.0*ESUM/EBEV(ISUBGT)
      SUMFR=SUMFR+FRACT
      SUMHOL=SUMHOL+PSUM
      AVERE=ESUM/PSUM
      IF(IMOUT(1).GT.0)
     1 WRITE(OUTP,100) AVERE,PSUM,FRACT,(SHBCD1(K),K=1,4)
   60 CONTINUE
      IF(IMOUT(1).GT.0) WRITE(OUTP,110) SUMHOL,SUMFR
      RETURN
   70 FORMAT(12A1,1X,2A1,1X,2A1,1X,2A1,1PE12.5,1PE12.5,1PE12.5,
     1 0PF12.7,I6)
   80 FORMAT(1X,72('=')/' Bound-Bound X-Ray Spectrum'
     1 ' For Each Shell Summed Over Subshells'/1X,72('=')/
     2 ' Average Emitted      Probability per    Per-cent of',
     3 '     Transition'/
     4 '         Energy (eV)  Initial Vacancy    Binding Energy'/
     5 1X,72('='))
   90 FORMAT(1X,72('=')/' Bound-Bound X-Ray Spectrum'
     1 ' For Each Shell Summed Over All Transitions'/1X,72('=')/
     2 ' Average Emitted      Probability per    Per-cent of',
     3 '     Transition'/
     4 '         Energy (eV)  Initial Vacancy    Binding Energy'/
     5 1X,72('='))
  100 FORMAT(6X,2X,F11.2,2X,F11.8,6X,F11.4,8X,4A1,4A1,4A1)
  110 FORMAT(16X,'Sum',2X,F11.8,6X,F11.4)
      END
      SUBROUTINE LIST2
C=======================================================================
C
C     OUTPUT SHELL FLUORESCENCE YIELD
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*1 SHBCD,MYNAME
      INTEGER OUTP,OTAPE
      COMMON/UNITS/INP,OUTP,ITAPE,OTAPE,KTAPE,LTAPE,
     1 IPLOT1,IPLOT2,IPLOT3,IPLOT4
      COMMON/FLUORS/FLUOR(100,39,6),MSHELL,MSUB1,MSUB2
      COMMON/OUTOPS/IMOUT(6)
      DIMENSION SHBCD(12),MYNAME(7),MYSUB(2,7),SHELL(100),FNORM(100)
      DATA MYNAME/'K','L','M','N','O','P','Q'/
C-----SUBSHELL DESIGNATOR RANGES FOR EACH SHELL.
      DATA MYSUB/
     1    1,    1,
     2    3,    6,
     3    8,   14,
     4   16,   25,
     5   27,   39,
     6   41,   56,
     7   58,   61/
      DATA ZERO/0.0D+00/
      DATA ONE/1.0D+00/
C-----NOTHING TO DO IF OUTPUT NOT REQUESTED.
      IF(IMOUT(4).LE.0) RETURN
C
C     BY SHELL.
C
      DO 80 MSHELL=1,7
      KSUB1=ISDOWN(MYSUB(1,MSHELL))
      KSUB2=ISDOWN(MYSUB(2,MSHELL))
      DO 70 INDEX=1,6
C-----INITIALIZE.
      DO 10 IZ=1,100
      FNORM(IZ)=ZERO
   10 SHELL(IZ)=ZERO
C-----SUM SUBSHELLS IN SHELL.
      DO 50 KSUB=KSUB1,KSUB2
      DO 20 IZ=1,100
      IF(FLUOR(IZ,KSUB,INDEX).GT.ZERO) GO TO 30
   20 CONTINUE
      GO TO 50
   30 DO 40 KZ=IZ,100
      FNORM(KZ)=FNORM(KZ)+ONE
   40 SHELL(KZ)=SHELL(KZ)+FLUOR(KZ,KSUB,INDEX)
   50 CONTINUE
C-----START OUTPUT AT FIRST NON-ZERO VALUE.
      CALL RANGER(SHELL,IZ1,IZ2,100)
      IF(IZ2.LE.0) GO TO 70
      IF(INDEX.EQ.1) WRITE(IPLOT4,160) MYNAME(MSHELL)
      IF(INDEX.EQ.2) WRITE(IPLOT4,170) MYNAME(MSHELL)
      IF(INDEX.EQ.3) WRITE(IPLOT4,180) MYNAME(MSHELL)
      IF(INDEX.EQ.4) WRITE(IPLOT4,160) MYNAME(MSHELL)
      IF(INDEX.EQ.5) WRITE(IPLOT4,170) MYNAME(MSHELL)
      IF(INDEX.EQ.6) WRITE(IPLOT4,180) MYNAME(MSHELL)
      DO 60 KZ=IZ1,IZ2
      ZK=KZ
   60 CALL POINT11(IPLOT4,ZK,SHELL(KZ),DUM,DUM,1,1,0,0)
      WRITE(IPLOT4,150)
   70 CONTINUE
   80 CONTINUE
C
C     BY SUBSHELL.
C
      DO 110 J=1,39
      DO 100 I=1,6
      CALL RANGER(FLUOR(1,J,I),IZ1,IZ2,100)
      IF(IZ2.LE.0) GO TO 100
      ISUB=ISUP(J)
      CALL SUBHOL(ISUB,SHBCD)
      IF(I.EQ.1) WRITE(IPLOT1,120) SHBCD
      IF(I.EQ.2) WRITE(IPLOT1,130) SHBCD
      IF(I.EQ.3) WRITE(IPLOT1,140) SHBCD
      IF(I.EQ.4) WRITE(IPLOT1,120) SHBCD
      IF(I.EQ.5) WRITE(IPLOT1,130) SHBCD
      IF(I.EQ.6) WRITE(IPLOT1,140) SHBCD
      DO 90 M=IZ1,IZ2
      ZM=M
   90 CALL POINT11(IPLOT1,ZM,FLUOR(M,J,I),DUM,DUM,1,1,0,0)
      WRITE(IPLOT1,150)
  100 CONTINUE
  110 CONTINUE
      RETURN
  120 FORMAT(1X,12A1,' Direct  ')
  130 FORMAT(1X,12A1,' Enhanced')
  140 FORMAT(1X,12A1,' Total   ')
  150 FORMAT(30X,'(END OF CURVE)')
  160 FORMAT(1X,A1,' Direct')
  170 FORMAT(1X,A1,' Enhanced')
  180 FORMAT(1X,A1,' Total')
      END
      SUBROUTINE USER1
C=======================================================================
C
C     USER INTERFACE ROUTINE. CALLED AFTER EACH SUBSHELL.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*1 ZABCD,SHBCD
      CHARACTER*1 FIELD1(11),FIELD2(11)
      INTEGER OUTP,OTAPE
      COMMON/UNITS/INP,OUTP,ITAPE,OTAPE,KTAPE,LTAPE,
     1 IPLOT1,IPLOT2,IPLOT3,IPLOT4
      COMMON/WHERE/ATWTGT,IZGET,ISUBGT,MAXSUB,MAXTRA
      COMMON/WHEREC/ZABCD(12),SHBCD(12)
      COMMON/SPECTR/ERO(5000),PROBRO(5000),NR
      COMMON/SPECTN/ENO(5000),PROBNO(5000),NN
      COMMON/SPECTL/ELO(39),PROBLO(39),NL
      COMMON/EADL/EBMEV(39),EBEV(39),ELECT(39),HOLES(39)
      COMMON/OUTER/OUTR(39),OUTN(39),OUTL(39),OUTRE(39),OUTNE(39),
     1 OUTLE(39),OUTAVR(39),OUTAVN(39),OUTAVL(39)
      COMMON/OUTOPS/IMOUT(6)
      DATA ZERO/0.0D+00/
      DATA ONE/1.0D+00/
      DATA XPLOT/1.35D+01/
      DATA YPLOT/1.0D+01/
      DATA IPASS/0/
      DATA IZERO/0/
      DATA IONE/1/
      DATA ITHREE/3/
C
C     SPECTRA AND INTEGRAL - PLOTTAB FORMATTED OUTPUT
C
C     NOTHING TO DO IF PLOTTAB OUTPUT NOT REQUESTED.
C
      IF(IMOUT(3).LE.0) RETURN
C
C     NO PLOTTAB OUTPUT UNLESS ALL ARE PRESENT.
C
      IF(NR.LE.0.OR.NN.LE.0.OR.NL.LE.0) RETURN
C
C     OUTPUT PLOT LAYOUT TO START.
C
      IF(IPASS.NE.0) GO TO 10
      IPASS=1
      WRITE(IPLOT3,100) ZERO,XPLOT,ZERO,YPLOT,IONE,IONE
      WRITE(IPLOT3,110) ITHREE,ITHREE,IZERO,IZERO,IZERO,IZERO
      WRITE(IPLOT3,120)
C-----DESCRIPTION OF ELEMENT AND SUBSHELL...X RANGE 10 EV TO BINDING
C-----ENERGY...Y RANGE 1.0E-6 TO MAXIMUM.
   10 CALL OUT9(EBEV(ISUBGT),FIELD1(1))
      CALL OUT9(ONE         ,FIELD2(1))
      WRITE(IPLOT3,20) ZABCD,SHBCD,EBEV(ISUBGT),FIELD2,
     1 FIELD1,(IZERO,I=1,8)
   20 FORMAT(12A1,12A1,F10.2,' eV Binding Energy'/
     1 ' Atom Relaxation Spectra and Normalized Integrals'/
     2 11A1,11A1,4I11/22X,4I11)
C-----FRACTION OF ENERGY TO RADIATIVE/NON-RADIATIVE/LOCAL.
      FRACTR=100.0*OUTRE(ISUBGT)/EBEV(ISUBGT)
      FRACTN=100.0*OUTNE(ISUBGT)/EBEV(ISUBGT)
      FRACTL=100.0*OUTLE(ISUBGT)/EBEV(ISUBGT)
C
C     SPECTRA AND INTEGRAL OF SPECTRA.
C
C-----X-RAYS.
      WRITE(IPLOT1,60) FRACTR
      WRITE(IPLOT2,60) FRACTR
      SUMP=ZERO
      DO 30 I=1,NR
      IF(ERO(I).LE.ZERO) GO TO 30
      IF(SUMP.GT.ZERO)
     1 CALL POINT11(IPLOT1,ERO(I),SUMP,DUM,DUM,1,1,0,0)
      SUMP=SUMP+ERO(I)*PROBRO(I)/EBEV(ISUBGT)
      CALL POINT11(IPLOT1,ERO(I),SUMP,DUM,DUM,1,1,0,0)
      CALL POINT11(IPLOT2,ERO(I),DUM,DUM,PROBRO(I),1,0,0,1)
   30 CONTINUE
      CALL POINT11(IPLOT1,EBEV(ISUBGT),SUMP,DUM,DUM,1,1,0,0)
      WRITE(IPLOT1,90)
      WRITE(IPLOT2,90)
C-----ELECTRONS
      WRITE(IPLOT1,70) FRACTN
      WRITE(IPLOT2,70) FRACTN
      SUMP=ZERO
      DO 40 I=1,NN
      IF(ENO(I).LE.ZERO) GO TO 40
      IF(SUMP.GT.ZERO)
     1 CALL POINT11(IPLOT1,ENO(I),SUMP,DUM,DUM,1,1,0,0)
      SUMP=SUMP+ENO(I)*PROBNO(I)/EBEV(ISUBGT)
      CALL POINT11(IPLOT1,ENO(I),SUMP,DUM,DUM,1,1,0,0)
      CALL POINT11(IPLOT2,ENO(I),DUM,DUM,PROBNO(I),1,0,0,1)
   40 CONTINUE
      CALL POINT11(IPLOT1,EBEV(ISUBGT),SUMP,DUM,DUM,1,1,0,0)
      WRITE(IPLOT1,90)
      WRITE(IPLOT2,90)
C-----FREE-BOUND
      WRITE(IPLOT1,80) FRACTL
      WRITE(IPLOT2,80) FRACTL
      SUMP=ZERO
      DO 50 I=1,NL
      IF(ELO(I).LE.ZERO) GO TO 50
      IF(SUMP.GT.ZERO)
     1 CALL POINT11(IPLOT1,ELO(I),SUMP,DUM,DUM,1,1,0,0)
      SUMP=SUMP+ELO(I)*PROBLO(I)/EBEV(ISUBGT)
      CALL POINT11(IPLOT1,ELO(I),SUMP,DUM,DUM,1,1,0,0)
      CALL POINT11(IPLOT2,ELO(I),DUM,DUM,PROBLO(I),1,0,0,1)
   50 CONTINUE
      CALL POINT11(IPLOT1,EBEV(ISUBGT),SUMP,DUM,DUM,1,1,0,0)
      WRITE(IPLOT1,90)
      WRITE(IPLOT2,90)
      RETURN
   60 FORMAT('X-Rays   ',F8.4,' %')
   70 FORMAT('Electrons',F8.4,' %')
   80 FORMAT('Local    ',F8.4,' %')
   90 FORMAT('                          ')
  100 FORMAT(4F11.5,2I11)
  110 FORMAT(6I11)
  120 FORMAT(' Emission Energy (eV)'/
     1 ' Spectra and Normalized Integrals')
      END
      SUBROUTINE USER2
C=======================================================================
C
C     USER INTERFACE ROUTINE. CALLED AFTER EACH ELEMENT.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      REAL*8 MEV2EV
      CHARACTER*1 FIELD1(11),FIELD2(11),FIELD3(11)
      INTEGER OUTP,OTAPE
      COMMON/UNITS/INP,OUTP,ITAPE,OTAPE,KTAPE,LTAPE,
     1 IPLOT1,IPLOT2,IPLOT3,IPLOT4
      COMMON/WHERE/ATWTGT,IZGET,ISUBGT,MAXSUB,MAXTRA
      COMMON/TRANS/TR(39,39),TNR(39,39,39),NSUBR(39),NSUBN(39)
      COMMON/EADL/EBMEV(39),EBEV(39),ELECT(39),HOLES(39)
      COMMON/OUTER/OUTR(39),OUTN(39),OUTL(39),OUTRE(39),OUTNE(39),
     1 OUTLE(39),OUTAVR(39),OUTAVN(39),OUTAVL(39)
      COMMON/OUTOPS/IMOUT(6)
      DATA ZERO/0.0D+00/
      DATA MEV2EV/1.0D+06/
      DATA IDATE/901205/
C
C     ENDL FORMATTED OUTPUT.
C
C     NOTHING TO DO IF ENDL FORMATTED OUTPUT NOT REQUESTED.
C
      IF(IMOUT(2).LE.0) RETURN
      IZA=1000*IZGET
      CALL OUT9(ATWTGT,FIELD1(1))
      CALL OUT9(ZERO  ,FIELD2(1))
C
C     ELECTRONS PER SUBSHELL.
C
      WRITE(OTAPE,10) IZA,FIELD1,IDATE,(FIELD2,K=1,3)
      WRITE(OTAPE,20) (FIELD2,K=1,5)
   10 FORMAT(I6,'  0  0',1X,11A1,I7,'  2',3(1X,11A1))
   20 FORMAT('91912  0',5(1X,11A1))
      DO 30 I=1,MAXSUB
      IF(ELECT(I).LE.ZERO) GO TO 30
      SUB=ISUP(I)
      CALL POINT16(OTAPE,SUB,ELECT(I),DUM,DUM,1,1,0,0)
   30 CONTINUE
      WRITE(OTAPE,290)
C
C     BINDING ENERGY.
C
      WRITE(OTAPE,10) IZA,FIELD1,IDATE,(FIELD2,K=1,3)
      WRITE(OTAPE,40) (FIELD2,K=1,5)
   40 FORMAT('91913  0',5(1X,11A1))
      DO 50 I=1,MAXSUB
      IF(EBMEV(I).LE.ZERO) GO TO 50
      SUB=ISUP(I)
      CALL POINT16(OTAPE,SUB,EBMEV(I),DUM,DUM,1,1,0,0)
   50 CONTINUE
      WRITE(OTAPE,290)
C
C     LOCAL ENERGY. SKIP IF ALL ARE ZERO.
C
      CALL RANGER(OUTLE,IZ1,IZ2,39)
      IF(IZ2.LE.0) GO TO 80
      WRITE(OTAPE,10) IZA,FIELD1,IDATE,
     1 (FIELD2,K=1,3)
      WRITE(OTAPE,60) (FIELD2,K=1,5)
   60 FORMAT('92935  0',5(1X,11A1))
      DO 70 I=IZ1,IZ2
      IF(EBMEV(I).LE.ZERO) GO TO 70
      IF(OUTLE(I).LE.ZERO) GO TO 70
      SUB=ISUP(I)
      XX=OUTLE(I)/MEV2EV
      CALL POINT16(OTAPE,SUB,XX,DUM,DUM,1,1,0,0)
   70 CONTINUE
      WRITE(OTAPE,290)
C
C     X-RAYS
C
C-----NUMBER OF X-RAYS. SKIP IF ALL ARE ZERO.
   80 CALL RANGER(OUTR,IZ1,IZ2,39)
      IF(IZ2.LE.0) GO TO 140
      WRITE(OTAPE,90) IZA,FIELD1,IDATE,
     1 (FIELD2,K=1,3)
      WRITE(OTAPE,100) (FIELD2,K=1,5)
   90 FORMAT(I6,'  0  7',1X,11A1,I7,'  2',3(1X,11A1))
  100 FORMAT('92933  0',5(1X,11A1))
      DO 110 I=IZ1,IZ2
      IF(EBMEV(I).LE.ZERO) GO TO 110
      IF(OUTR(I).LE.ZERO) GO TO 110
      SUB=ISUP(I)
      CALL POINT16(OTAPE,SUB,OUTR(I),DUM,DUM,1,1,0,0)
  110 CONTINUE
      WRITE(OTAPE,290)
C-----ENERGY.
      WRITE(OTAPE,90) IZA,FIELD1,IDATE,
     1 (FIELD2,K=1,3)
      WRITE(OTAPE,120) (FIELD2,K=1,5)
  120 FORMAT('92934  0',5(1X,11A1))
      DO 130 I=IZ1,IZ2
      IF(EBMEV(I).LE.ZERO) GO TO 130
      IF(OUTRE(I).LE.ZERO) GO TO 130
      SUB=ISUP(I)
      XX=OUTRE(I)/MEV2EV
      CALL POINT16(OTAPE,SUB,XX,DUM,DUM,1,1,0,0)
  130 CONTINUE
      WRITE(OTAPE,290)
C
C     ELECTRONS
C
C-----NUMBER OF ELECTRONS. SKIP IF ALL ARE ZERO.
  140 CALL RANGER(OUTN,IZ1,IZ2,39)
      IF(IZ2.LE.0) GO TO 180
      WRITE(OTAPE,150) IZA,FIELD1,IDATE,
     1 (FIELD2,K=1,3)
      WRITE(OTAPE,100) (FIELD2,K=1,5)
  150 FORMAT(I6,'  0  9',1X,11A1,I7,'  2',3(1X,11A1))
      DO 160 I=IZ1,IZ2
      IF(EBMEV(I).LE.ZERO) GO TO 160
      IF(OUTN(I).LE.ZERO) GO TO 160
      SUB=ISUP(I)
      CALL POINT16(OTAPE,SUB,OUTN(I),DUM,DUM,1,1,0,0)
  160 CONTINUE
      WRITE(OTAPE,290)
C-----ENERGY.
      WRITE(OTAPE,150) IZA,FIELD1,IDATE,
     1 (FIELD2,K=1,3)
      WRITE(OTAPE,120) (FIELD2,K=1,5)
      DO 170 I=IZ1,IZ2
      IF(EBMEV(I).LE.ZERO) GO TO 170
      IF(OUTNE(I).LE.ZERO) GO TO 170
      SUB=ISUP(I)
      XX=OUTNE(I)/MEV2EV
      CALL POINT16(OTAPE,SUB,XX,DUM,DUM,1,1,0,0)
  170 CONTINUE
      WRITE(OTAPE,290)
C
C     PRIMARY TRANSITIONS AND ENERGY.
C
  180 DO 280 ISUB=1,MAXSUB
      IF(EBMEV(ISUB).LE.ZERO) GO TO 280
      SUBI=ISUP(ISUB)
      CALL OUT9 (SUBI,FIELD3(1))
C-----RADIATIVE. SKIP IF ALL ARE ZERO.
      IF(NSUBR(ISUB).LE.0) GO TO 230
      DO 190 KSUB=1,MAXSUB
      IF(TR(ISUB,KSUB).GT.ZERO) GO TO 200
  190 CONTINUE
      GO TO 230
  200 WRITE(OTAPE,90) IZA,FIELD1,IDATE,
     1 (FIELD2,K=1,3)
      WRITE(OTAPE,210) FIELD2,FIELD3,
     1 (FIELD2,K=3,5)
  210 FORMAT('92931 91',5(1X,11A1))
      DO 220 KSUB=1,MAXSUB
      IF(TR(ISUB,KSUB).LE.ZERO) GO TO 220
      SUBK=ISUP(KSUB)
      ET=EBMEV(ISUB)-EBMEV(KSUB)
      IF(ET.LT.ZERO) ET=ZERO
      CALL POINT16(OTAPE,SUBK,TR(ISUB,KSUB),ET,DUM,1,1,1,0)
  220 CONTINUE
      WRITE(OTAPE,290)
C-----NON-RADIATIVE. SKIP IF ALL ARE ZERO.
  230 IF(NSUBN(ISUB).LE.0) GO TO 280
      DO 240 KSUB=1,MAXSUB
      DO 240 LSUB=1,MAXSUB
      IF(TNR(ISUB,KSUB,LSUB).GT.ZERO) GO TO 250
  240 CONTINUE
      GO TO 280
  250 WRITE(OTAPE,150) IZA,FIELD1,IDATE,
     1 (FIELD2,K=1,3)
      WRITE(OTAPE,260) FIELD2,FIELD3,
     1 (FIELD2,K=3,5)
  260 FORMAT('92932 91',5(1A,11A1))
      DO 270 KSUB=1,MAXSUB
      DO 270 LSUB=1,MAXSUB
      IF(TNR(ISUB,KSUB,LSUB).LE.ZERO) GO TO 270
      SUBK=ISUP(KSUB)
      SUBL=ISUP(LSUB)
      ET=EBMEV(ISUB)-(EBMEV(KSUB)+EBMEV(LSUB))
      IF(ET.LT.ZERO) ET=ZERO
      CALL POINT16(OTAPE,SUBK,SUBL,TNR(ISUB,KSUB,LSUB),ET,1,1,1,1)
  270 CONTINUE
      WRITE(OTAPE,290)
  280 CONTINUE
      RETURN
  290 FORMAT(71X,'1')
      END
      SUBROUTINE POINT11(IUNIT,Z1,Z2,Z3,Z4,I1,I2,I3,I4)
C=======================================================================
C
C     11 COLUMNS - USED FOR PLOTTAB FILES.
C
C     OUTPUT ONE POINT (A LINE) WITH UP TO 4 FIELD
C
C     TYPES OF LAYOUT CONSIDERED INCLUDE,
C     1 2
C     1 2 3
C     1 2 3 4
C     1     4
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*1 FIELD1(11),FIELD2(11),FIELD3(11),FIELD4(11)
C-----ERROR IF FIELD 1 IS NOT SELECTED.
      IF(I1.LE.0) RETURN
C-----ALWAYS USE FIELD 1
C-----SELECT FORMAT
      IF(I2.EQ.0) GO TO 30
      IF(I3.NE.0) GO TO 10
C----- 1 2
      CALL OUT9(Z1,FIELD1(1))
      CALL OUT9(Z2,FIELD2(1))
      WRITE(IUNIT,50) FIELD1,FIELD2
      GO TO 40
   10 IF(I4.NE.0) GO TO 20
C----- 1 2 3
      CALL OUT9(Z1,FIELD1(1))
      CALL OUT9(Z2,FIELD2(1))
      CALL OUT9(Z3,FIELD3(1))
      WRITE(IUNIT,50) FIELD1,FIELD2,FIELD3
      GO TO 40
C----- 1 2 3 4
   20 CALL OUT9(Z1,FIELD1(1))
      CALL OUT9(Z2,FIELD2(1))
      CALL OUT9(Z3,FIELD3(1))
      CALL OUT9(Z4,FIELD4(1))
      WRITE(IUNIT,50) FIELD1,FIELD2,FIELD3,FIELD4
      GO TO 40
C----- 1     4
   30 CALL OUT9(Z1,FIELD1(1))
      CALL OUT9(Z4,FIELD4(1))
      WRITE(IUNIT,60) FIELD1,FIELD4
   40 RETURN
   50 FORMAT(22A1)
   60 FORMAT(11A1,22X,11A)
      END
      SUBROUTINE POINT16(IUNIT,Z1,Z2,Z3,Z4,I1,I2,I3,I4)
C=======================================================================
C
C     16 COLUMNS - USED FOR ENDLX FILES.
C
C     OUTPUT ONE POINT (A LINE) WITH UP TO 4 FIELD
C
C     TYPES OF LAYOUT CONSIDERED INCLUDE,
C     1 2
C     1 2 3
C     1 2 3 4
C     1     4
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
C-----ERROR IF FIELD 1 IS NOT SELECTED.
      IF(I1.LE.0) RETURN
C-----ALWAYS USE FIELD 1
C-----SELECT FORMAT
      IF(I2.EQ.0) GO TO 30
      IF(I3.NE.0) GO TO 10
C----- 1 2
      WRITE(IUNIT,50) Z1,Z2
      GO TO 40
   10 IF(I4.NE.0) GO TO 20
C----- 1 2 3
      WRITE(IUNIT,50) Z1,Z2,Z3
      GO TO 40
C----- 1 2 3 4
   20 WRITE(IUNIT,50) Z1,Z2,Z3,Z4
      GO TO 40
C----- 1     4
   30 WRITE(IUNIT,60) Z1,Z4
   40 RETURN
   50 FORMAT(1P4E16.9)
   60 FORMAT(1PE16.9,32X,1PE16.9)
      END
      SUBROUTINE SPECTA
C=======================================================================
C
C     DEFINE SPECTRUM OF EMITTED X-RAYS AND ELECTRONS.
C
C     RETURN ATOM TO NEUTRALITY
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      COMMON/WHERE/ATWTGT,IZGET,ISUBGT,MAXSUB,MAXTRA
      COMMON/TRANS/TR(39,39),TNR(39,39,39),NSUBR(39),NSUBN(39)
      COMMON/SPECTR/ERO(5000),PROBRO(5000),NR
      COMMON/SPECTN/ENO(5000),PROBNO(5000),NN
      COMMON/SPECTL/ELO(39),PROBLO(39),NL
      COMMON/EADL/EBMEV(39),EBEV(39),ELECT(39),HOLES(39)
      COMMON/OUTER/OUTR(39),OUTN(39),OUTL(39),OUTRE(39),OUTNE(39),
     1 OUTLE(39),OUTAVR(39),OUTAVN(39),OUTAVL(39)
      COMMON/INDEXS/INDEXR(5000),INDEXN(5000),INDEXL(39)
      COMMON/FLUORS/FLUOR(100,39,6),MSHELL,MSUB1,MSUB2
      DATA ZERO/0.0D+00/
      DATA ONE/1.0D+00/
      DATA HOLMIN/1.0D-10/
C-----NOTHING TO DO IF SUBSHELL ENERGY IS NOT DEFINED.
      IF(EBMEV(ISUBGT).LE.ZERO) RETURN
C-----INITIALIZE NUMBER OF HOLES.
      DO 10 I=ISUBGT,MAXSUB
   10 HOLES(I)=ZERO
      HOLES(ISUBGT)=ONE
      HOLEX=ONE
      IF(ELECT(ISUBGT).LT.ONE) HOLEX=ELECT(ISUBGT)
      HOLES(ISUBGT)=HOLEX
      HNORM=ONE/HOLEX
C
C     LOOP OVER SUBSHELLS.
C
      DO 60 ISUB=ISUBGT,MAXSUB
C-----ONLY CONSIDER SUBSHELLS FOR WHICH HOLES EXIST AND DATA HAS
C-----BEEN READ.
      IF(HOLES(ISUB).EQ.ZERO) GO TO 60
C
C     FILL INITIAL HOLE AND GENERATE X-RAYS AND ELECTRONS.
C
C     RADIATIVE.
C
C-----DEFINE INITIAL NUMBER OF HOLES.
      HOLEK=HOLES(ISUB)
      IF(NSUBR(ISUB).LE.0) GO TO 40
C-----LOOP OVER TRANSITIONS.
      DO 30 I=ISUB,MAXSUB
      IF(TR(ISUB,I).LE.ZERO) GO TO 30
C-----FILL AND CREATE HOLE.
      HOLEI=HOLEK*TR(ISUB,I)
      HOLE1=HNORM*HOLEI
      HOLES(ISUB)=HOLES(ISUB)-HOLEI
      HOLES(I)=HOLES(I)+HOLEI
C-----FLUORESCENCE YIELD - TRANSITIONS.
      FLUOR(IZGET,ISUBGT,3)=FLUOR(IZGET,ISUBGT,3)+HOLEI
      IF(ISUB.GT.MSUB2) GO TO 20
      FLUOR(IZGET,ISUBGT,2)=FLUOR(IZGET,ISUBGT,2)+HOLEI
      IF(ISUB.NE.ISUBGT) GO TO 20
      FLUOR(IZGET,ISUBGT,1)=FLUOR(IZGET,ISUBGT,1)+HOLEI
C-----EMIT X-RAY.
   20 NR=NR+1
      IF(NR.GT.5000) GO TO 150
      ERO(NR)=EBEV(ISUB)-EBEV(I)
      PROBRO(NR)=HOLE1
      INDEXR(NR)=100*ISUB+I
      IF(ERO(NR).LT.ZERO) ERO(NR)=ZERO
C-----FLUORESCENCE YIELD - ENERGY.
      EHOLEI=ERO(NR)*HOLEI
      FLUOR(IZGET,ISUBGT,6)=FLUOR(IZGET,ISUBGT,6)+EHOLEI
      IF(ISUB.GT.MSUB2) GO TO 30
      FLUOR(IZGET,ISUBGT,5)=FLUOR(IZGET,ISUBGT,5)+EHOLEI
      IF(ISUB.NE.ISUBGT) GO TO 30
      FLUOR(IZGET,ISUBGT,4)=FLUOR(IZGET,ISUBGT,4)+EHOLEI
C-----EMIT X-RAY.
   30 CONTINUE
C
C     NON-RADIATIVE.
C
   40 IF(NSUBN(ISUB).LE.0) GO TO 60
C-----LOOP OVER TRANSITIONS.
      DO 50 I=ISUB,MAXSUB
      DO 50 J=ISUB,MAXSUB
      IF(TNR(ISUB,I,J).LE.ZERO) GO TO 50
C-----FILL AND CREATE 2 HOLES.
      HOLEIJ=HOLEK*TNR(ISUB,I,J)
      HOLE1=HNORM*HOLEIJ
      HOLES(ISUB)=HOLES(ISUB)-HOLEIJ
      HOLES(I)=HOLES(I)+HOLEIJ
      HOLES(J)=HOLES(J)+HOLEIJ
C-----EMIT ELECTRON.
      NN=NN+1
      IF(NN.GT.5000) GO TO 160
      ENO(NN)=EBEV(ISUB)-(EBEV(I)+EBEV(J))
      PROBNO(NN)=HOLE1
      INDEXN(NN)=10000*ISUB+100*I+J
      IF(ENO(NN).LT.ZERO) ENO(NN)=ZERO
   50 CONTINUE
C
C     NEXT SUBSHELL.
C
   60 CONTINUE
C
C     LOCAL = FREE-BOUND TRANSITIONS...BASED ON REMAINING HOLES
C     AND SUBSHELL BINDING ENERGIES.
C
      DO 70 ISUB=MAXSUB,ISUBGT,-1
      IF(HOLES(ISUB).LE.HOLMIN) GO TO 70
      NL=NL+1
      ELO(NL)=EBEV(ISUB)
      PROBLO(NL)=HNORM*HOLES(ISUB)
      INDEXL(NL)=ISUB
   70 CONTINUE
C
C     SORT SPECTRA INTO ASCENDING ENERGY ORDER.
C
      CALL SORTS(ERO,PROBRO,INDEXR,NR)
      CALL SORTS(ENO,PROBNO,INDEXN,NN)
      CALL SORTS(ELO,PROBLO,INDEXL,NL)
C
C     DEFINE NUMBER OF PARTICLES AND ENERGY EMITTED.
C
C-----X-RAYS.
      SUMR=ZERO
      ERAD=ZERO
      IF(NR.LE.0) GO TO 90
      DO 80 I=1,NR
      SUMR=SUMR+PROBRO(I)
   80 ERAD=ERAD+PROBRO(I)*ERO(I)
C-----ELECTRONS.
   90 SUMN=ZERO
      ENORAD=ZERO
      IF(NN.LE.0) GO TO 110
      DO 100 I=1,NN
      SUMN=SUMN+PROBNO(I)
  100 ENORAD=ENORAD+PROBNO(I)*ENO(I)
C-----LOCAL.
  110 SUML=ZERO
      ELOC1=ZERO
      ELOCAL=ZERO
      IF(NL.LE.0) GO TO 140
      DO 120 I=1,NL
      SUML=SUML+PROBLO(I)
  120 ELOC1=ELOC1+PROBLO(I)*ELO(I)
C
C     RE-BALANCE TO OBTAIN EXACT ENERGY BALANCE (ENERGY WILL NOT
C     BALANCE ONLY IN THE CASE WHERE THE TRANSITIONS INDICATE A
C     NEGATIVE ENERGY ELECTRON EMITTED AND ITS ENERGY HAS BEEN
C     SET TO ZERO, ABOVE...THIS WILL RESULT IN TOO MUCH ENERGY
C     REMAINING IN THE ATOM...BASED ON REMAINING HOLES AND THE
C     BINDING ENERGY OF THE SUBSHELLS IN WHICH HOLES REMAIN.
C     THE NUMBER OF HOLES CANNOT BE CHANGED IF THE ATOM IS TO
C     RETURN TO NEUTRALITY AND ELECTRONS CONSERVED. RE-BALANCE
C     IS ACHIEVED BY SLIGHTLY CHANGING THE ENERGY OF FREE-BOUND
C     TRANSITIONS.
C
      ELOCAL=EBEV(ISUBGT)-(ERAD+ENORAD)
      RATLOC=ELOCAL/ELOC1
      DO 130 I=1,NL
  130 ELO(I)=RATLOC*ELO(I)
C
C     SAVE AVERAGE VALUES.
C
C-----SAVE NUMBER OF X-RAYS, ELECTRONS AND FREE-BOUND EMISSIONS.
  140 OUTR(ISUBGT)=SUMR
      OUTN(ISUBGT)=SUMN
      OUTL(ISUBGT)=SUML
C-----SAVE ENERGY.
      OUTRE(ISUBGT)=ERAD
      OUTNE(ISUBGT)=ENORAD
      OUTLE(ISUBGT)=ELOCAL
C-----SAVE AVERAGE EMISSION ENERGIES.
      OUTAVR(ISUBGT)=ZERO
      OUTAVN(ISUBGT)=ZERO
      OUTAVL(ISUBGT)=ZERO
      IF(SUMR.GT.ZERO) OUTAVR(ISUBGT)=ERAD/SUMR
      IF(SUMN.GT.ZERO) OUTAVN(ISUBGT)=ENORAD/SUMN
      IF(SUML.GT.ZERO) OUTAVL(ISUBGT)=ELOCAL/SUML
      RETURN
C
C     SPECTRA CORE EXCEEDED.
C
  150 WRITE(6,170)
      STOP
  160 WRITE(6,180)
      STOP
  170 FORMAT(' RADIATIVE SPECTRUM CORE EXCEEDED')
  180 FORMAT(' NON-RADIATIVE SPECTRUM CORE EXCEEDED')
      END
      SUBROUTINE SPECTB
C=======================================================================
C
C     DEFINE SPECTRUM OF EMITTED X-RAYS AND ELECTRONS.
C
C     ISOLATED ATOM
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      COMMON/WHERE/ATWTGT,IZGET,ISUBGT,MAXSUB,MAXTRA
      COMMON/TRANS/TR(39,39),TNR(39,39,39),NSUBR(39),NSUBN(39)
      COMMON/SPECTR/ERO(5000),PROBRO(5000),NR
      COMMON/SPECTN/ENO(5000),PROBNO(5000),NN
      COMMON/SPECTL/ELO(39),PROBLO(39),NL
      COMMON/EADL/EBMEV(39),EBEV(39),ELECT(39),HOLES(39)
      COMMON/OUTER/OUTR(39),OUTN(39),OUTL(39),OUTRE(39),OUTNE(39),
     1 OUTLE(39),OUTAVR(39),OUTAVN(39),OUTAVL(39)
      COMMON/INDEXS/INDEXR(5000),INDEXN(5000),INDEXL(39)
      COMMON/FLUORS/FLUOR(100,39,6),MSHELL,MSUB1,MSUB2
      DIMENSION ENOW(39)
      DATA ZERO/0.0D+00/
      DATA ONE/1.0D+00/
      DATA HOLMIN/1.0D-10/
C-----NOTHING TO DO IF SUBSHELL ENERGY IS NOT DEFINED.
      IF(EBMEV(ISUBGT).LE.ZERO) RETURN
C-----INITIALIZE NUMBER OF HOLES.
      DO 10 I=ISUBGT,MAXSUB
      ENOW(I)=ELECT(I)
   10 HOLES(I)=ZERO
      HOLES(ISUBGT)=ONE
      HOLEX=ONE
      IF(ELECT(ISUBGT).LT.ONE) HOLEX=ELECT(ISUBGT)
      ENOW(ISUBGT)=ENOW(ISUBGT)-HOLEX
      HOLES(ISUBGT)=HOLEX
      HNORM=ONE/HOLEX
C
C     LOOP OVER SUBSHELLS.
C
      DO 60 ISUB=ISUBGT,MAXSUB
C-----ONLY CONSIDER SUBSHELLS FOR WHICH HOLES EXIST AND DATA HAS
C-----BEEN READ.
      IF(HOLES(ISUB).EQ.ZERO) GO TO 60
C
C     FILL INITIAL HOLE AND GENERATE X-RAYS AND ELECTRONS.
C
C     RADIATIVE.
C
C-----DEFINE INITIAL NUMBER OF HOLES.
      HOLEK=HOLES(ISUB)
      IF(NSUBR(ISUB).LE.0) GO TO 40
C-----LOOP OVER TRANSITIONS.
      DO 30 I=ISUB,MAXSUB
      IF(TR(ISUB,I).LE.ZERO) GO TO 30
C-----FILL AND CREATE HOLE UP TO THE NUMBER OF ELECTRONS AVAILABLE.
      HOLEI=HOLEK*TR(ISUB,I)
      IF(HOLEI.GT.ENOW(I)) HOLEI=ENOW(I)
      HOLE1=HNORM*HOLEI
      HOLES(ISUB)=HOLES(ISUB)-HOLEI
      HOLES(I)=HOLES(I)+HOLEI
      ENOW(ISUB)=ENOW(ISUB)+HOLEI
      ENOW(I)=ENOW(I)-HOLEI
C-----FLUORESCENCE YIELD - TRANSITIONS.
      FLUOR(IZGET,ISUBGT,3)=FLUOR(IZGET,ISUBGT,3)+HOLEI
      IF(ISUB.GT.MSUB2) GO TO 20
      FLUOR(IZGET,ISUBGT,2)=FLUOR(IZGET,ISUBGT,2)+HOLEI
      IF(ISUB.NE.ISUBGT) GO TO 20
      FLUOR(IZGET,ISUBGT,1)=FLUOR(IZGET,ISUBGT,1)+HOLEI
C-----EMIT X-RAY.
   20 NR=NR+1
      IF(NR.GT.5000) GO TO 150
      ERO(NR)=EBEV(ISUB)-EBEV(I)
      PROBRO(NR)=HOLE1
      INDEXR(NR)=100*ISUB+I
      IF(ERO(NR).LT.ZERO) ERO(NR)=ZERO
C-----FLUORESCENCE YIELD - ENERGY.
      EHOLEI=ERO(NR)*HOLEI
      FLUOR(IZGET,ISUBGT,6)=FLUOR(IZGET,ISUBGT,6)+EHOLEI
      IF(ISUB.GT.MSUB2) GO TO 30
      FLUOR(IZGET,ISUBGT,5)=FLUOR(IZGET,ISUBGT,5)+EHOLEI
      IF(ISUB.NE.ISUBGT) GO TO 30
      FLUOR(IZGET,ISUBGT,4)=FLUOR(IZGET,ISUBGT,4)+EHOLEI
C-----EMIT X-RAY.
   30 CONTINUE
C
C     NON-RADIATIVE.
C
   40 IF(NSUBN(ISUB).LE.0) GO TO 60
C-----LOOP OVER TRANSITIONS.
      DO 50 I=ISUB,MAXSUB
      DO 50 J=ISUB,MAXSUB
      IF(TNR(ISUB,I,J).LE.ZERO) GO TO 50
C-----FILL AND CREATE HOLE UP TO THE NUMBER OF ELECTRONS AVAILABLE.
      HOLEIJ=HOLEK*TNR(ISUB,I,J)
      IF(HOLEIJ.GT.ENOW(I)) HOLEIJ=ENOW(I)
      IF(HOLEIJ.GT.ENOW(J)) HOLEIJ=ENOW(J)
      IF(I.EQ.J.AND.HOLEIJ.GT.0.5*ENOW(I)) HOLEIJ=0.5*ENOW(I)
      HOLE1=HNORM*HOLEIJ
      HOLES(ISUB)=HOLES(ISUB)-HOLEIJ
      HOLES(I)=HOLES(I)+HOLEIJ
      HOLES(J)=HOLES(J)+HOLEIJ
      ENOW(ISUB)=ENOW(ISUB)+HOLEIJ
      ENOW(I)=ENOW(I)-HOLEIJ
      ENOW(J)=ENOW(J)-HOLEIJ
C-----EMIT ELECTRON.
      NN=NN+1
      IF(NN.GT.5000) GO TO 160
      ENO(NN)=EBEV(ISUB)-(EBEV(I)+EBEV(J))
      PROBNO(NN)=HOLE1
      INDEXN(NN)=10000*ISUB+100*I+J
      IF(ENO(NN).LT.ZERO) ENO(NN)=ZERO
   50 CONTINUE
C
C     NEXT SUBSHELL.
C
   60 CONTINUE
C
C     LOCAL = FREE-BOUND TRANSITIONS...BASED ON REMAINING HOLES
C     AND SUBSHELL BINDING ENERGIES.
C
      DO 70 ISUB=MAXSUB,ISUBGT,-1
      IF(HOLES(ISUB).LE.HOLMIN) GO TO 70
      NL=NL+1
      ELO(NL)=EBEV(ISUB)
      PROBLO(NL)=HNORM*HOLES(ISUB)
      INDEXL(NL)=ISUB
   70 CONTINUE
C
C     SORT SPECTRA INTO ASCENDING ENERGY ORDER.
C
      CALL SORTS(ERO,PROBRO,INDEXR,NR)
      CALL SORTS(ENO,PROBNO,INDEXN,NN)
      CALL SORTS(ELO,PROBLO,INDEXL,NL)
C
C     DEFINE NUMBER OF PARTICLES AND ENERGY EMITTED.
C
C-----X-RAYS.
      SUMR=ZERO
      ERAD=ZERO
      IF(NR.LE.0) GO TO 90
      DO 80 I=1,NR
      SUMR=SUMR+PROBRO(I)
   80 ERAD=ERAD+PROBRO(I)*ERO(I)
C-----ELECTRONS.
   90 SUMN=ZERO
      ENORAD=ZERO
      IF(NN.LE.0) GO TO 110
      DO 100 I=1,NN
      SUMN=SUMN+PROBNO(I)
  100 ENORAD=ENORAD+PROBNO(I)*ENO(I)
C-----LOCAL.
  110 SUML=ZERO
      ELOC1=ZERO
      ELOCAL=ZERO
      IF(NL.LE.0) GO TO 140
      DO 120 I=1,NL
      SUML=SUML+PROBLO(I)
  120 ELOC1=ELOC1+PROBLO(I)*ELO(I)
C
C     RE-BALANCE TO OBTAIN EXACT ENERGY BALANCE (ENERGY WILL NOT
C     BALANCE ONLY IN THE CASE WHERE THE TRANSITIONS INDICATE A
C     NEGATIVE ENERGY ELECTRON EMITTED AND ITS ENERGY HAS BEEN
C     SET TO ZERO, ABOVE...THIS WILL RESULT IN TOO MUCH ENERGY
C     REMAINING IN THE ATOM...BASED ON REMAINING HOLES AND THE
C     BINDING ENERGY OF THE SUBSHELLS IN WHICH HOLES REMAIN.
C     THE NUMBER OF HOLES CANNOT BE CHANGED IF THE ATOM IS TO
C     RETURN TO NEUTRALITY AND ELECTRONS CONSERVED. RE-BALANCE
C     IS ACHIEVED BY SLIGHTLY CHANGING THE ENERGY OF FREE-BOUND
C     TRANSITIONS.
C
      ELOCAL=EBEV(ISUBGT)-(ERAD+ENORAD)
      RATLOC=ELOCAL/ELOC1
      DO 130 I=1,NL
  130 ELO(I)=RATLOC*ELO(I)
C
C     SAVE AVERAGE VALUES.
C
C-----SAVE NUMBER OF X-RAYS, ELECTRONS AND FREE-BOUND EMISSIONS.
  140 OUTR(ISUBGT)=SUMR
      OUTN(ISUBGT)=SUMN
      OUTL(ISUBGT)=SUML
C-----SAVE ENERGY.
      OUTRE(ISUBGT)=ERAD
      OUTNE(ISUBGT)=ENORAD
      OUTLE(ISUBGT)=ELOCAL
C-----SAVE AVERAGE EMISSION ENERGIES.
      OUTAVR(ISUBGT)=ZERO
      OUTAVN(ISUBGT)=ZERO
      OUTAVL(ISUBGT)=ZERO
      IF(SUMR.GT.ZERO) OUTAVR(ISUBGT)=ERAD/SUMR
      IF(SUMN.GT.ZERO) OUTAVN(ISUBGT)=ENORAD/SUMN
      IF(SUML.GT.ZERO) OUTAVL(ISUBGT)=ELOCAL/SUML
      RETURN
C
C     SPECTRA CORE EXCEEDED.
C
  150 WRITE(6,170)
      STOP
  160 WRITE(6,180)
      STOP
  170 FORMAT(' RADIATIVE SPECTRUM CORE EXCEEDED')
  180 FORMAT(' NON-RADIATIVE SPECTRUM CORE EXCEEDED')
      END
      SUBROUTINE SORTS(X,Y,INDEX,NXY)
C=======================================================================
C
C     SORT AN ARRAY INTO ASCENDING FLOATING POINT ORDER.
C
C     ARGUMENTS
C     ---------
C     X      = ARRAY TO SORT (DIMENSION NXY)
C     Y      = COMPANION ARRAY (DIMENSION NXY)
C     INDEX  = COMPANION ARRAY (DIMENSION NXY)
C     NXY    = NUMBER OF ELEMENTS TO SORT
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      INTEGER SWITCH
      DIMENSION X(NXY),Y(NXY),INDEX(NXY)
C-----IF LESS THAN TWO (2) ELEMENTS NO SORT IS REQUIRED.
      IF(NXY.LT.2) RETURN
C-----SET INNER LOOP INDICES
      LTOP=NXY+1
C-----SET UP OUTER LOOP
      DO 30 IN=2,NXY
C-----INITIALIZE EXCHANGE SWITCH OFF.
      SWITCH=0
C-----SET UPPER INDEX TO INNER LOOP
      LTOP=LTOP-1
C-----SET LARGEST ELEMENT INDICATOR TO FIRST ELEMENT
      LBIG=1
C-----SET UP INNER LOOP
      DO 20 J=2,LTOP
C-----COMPARE ELEMENTS
      IF(X(LBIG).GT.X(J)) GO TO 10
C-----ELEMENTS ARE IN NUMERICAL ORDER. RESET INDEX TO LARGER ELEMENT.
      LBIG=J
      GO TO 20
C-----ELEMENTS ARE NOT IN NUMERICAL ORDER. SET INTERCHANGE SWITCH.
   10 SWITCH=1
C-----END OF INNER LOOP
   20 CONTINUE
C-----ARE ALL ELEMENTS ALREADY IN ORDER......
      IF(SWITCH.LE.0) RETURN
C-----NO. MOVE LARGEST ELEMENT TO TOP OF REMAINING TABLE
      DUMMY=X(LBIG)
      X(LBIG)=X(LTOP)
      X(LTOP)=DUMMY
      DUMMY=Y(LBIG)
      Y(LBIG)=Y(LTOP)
      Y(LTOP)=DUMMY
      IDUM=INDEX(LBIG)
      INDEX(LBIG)=INDEX(LTOP)
      INDEX(LTOP)=IDUM
   30 CONTINUE
      RETURN
      END
      SUBROUTINE ZAHOL(ZA,ZABCD)
C=======================================================================
C
C     GIVEN ANY ZA (1000*Z+A) THIS ROUTINE WILL DEFINE A 12 CHARACTER
C     EQUIVALENT IN ONE OF THE FOLLOWING 2 FORMS,
C
C     (1) Z GREATER THAN 0
C         ZZZ-SS-AAA  - ZZZ = Z
C                        SS = CHEMICCAL SYMBOL
C                       AAA = A
C
C     FOR EXAMPLE ZA = 92238 IS RETURNED AS 92-U -238
C
C     (2) Z EQUAL TO 0
C         12 CHARACTERS FOR ENDF/B SPECIAL MATERIALS
C
C     FOR EXAMPLE ZA = 302 IS RETURNED AS ZIRCALLOY-2
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      INTEGER ZA,Z,A
      CHARACTER*1 DUM1,DUM2,ZABCD,ZATAB,GOOF,MINUS,NATURE,BLANK,DIGITS
      DIMENSION ZATAB(2,103),DUM1(2,54),DUM2(2,49),ZABCD(12),GOOF(12),
     1 NATURE(4),DIGITS(10)
      EQUIVALENCE (ZATAB(1,1),DUM1(1,1)),(ZATAB(1,55),DUM2(1,1))
      DATA GOOF/'Z','A',' ','U','n','k','n','o','w','n',' ',' '/
      DATA MINUS/'-'/
      DATA NATURE/'N','a','t',' '/
      DATA BLANK/' '/
      DATA DIGITS/'0','1','2','3','4','5','6','7','8','9'/
      DATA DUM1/
     1 'H',' ','H','e','L','i','B','e','B',' ','C',' ',
     1 'N',' ','O',' ','F',' ','N','e','N','a','M','g',
     1 'A','l','S','i','P',' ','S',' ','C','l','A','r',
     1 'K',' ','C','a','S','c','T','i','V',' ','C','r',
     1 'M','n','F','e','C','o','N','i','C','u','Z','n',
     1 'G','a','G','e','A','s','S','e','B','r','K','r',
     1 'R','b','S','r','Y',' ','Z','r','N','b','M','o',
     1 'T','c','R','u','R','h','P','d','A','g','C','d',
     1 'I','n','S','n','S','b','T','e','I',' ','X','e'/
      DATA DUM2/
     1 'C','s','B','a','L','a','C','e','P','r','N','d',
     1 'P','m','S','m','E','u','G','d','T','b','D','y',
     1 'H','o','E','r','T','m','Y','b','L','u','H','f',
     1 'T','a','W',' ','R','e','O','s','I','r','P','t',
     1 'A','u','H','g','T','l','P','b','B','i','P','o',
     1 'A','t','R','n','F','r','R','a','A','c','T','h',
     1 'P','a','U',' ','N','p','P','u','A','m','C','m',
     1 'B','k','C','f','E','s','F','m','M','d','N','o',
     1 'L','r'/
C-----BLANK OUT ZABCD TO START.
      DO 10 I=1,12
   10 ZABCD(I)=BLANK
C-----DEFINE Z AND A SEPARATELY.
      Z=ZA/1000
      A=ZA-1000*Z
C-----INSURE Z IS IN LEGAL RANGE.
      IF(Z.GE.0.AND.Z.LE.103) GO TO 30
      DO 20 I=1,12
   20 ZABCD(I)=GOOF(I)
      RETURN
C-----DEFINE SYMBOL FOR ELEMENT.
   30 ZABCD(5)=MINUS
      ZABCD(6)=ZATAB(1,Z)
      ZABCD(7)=ZATAB(2,Z)
      ZABCD(8)=MINUS
C-----DEFINE Z FIRST DIGIT TO LAST.
      IDIV=100
      IMON=0
      II=2
      DO 50 I=1,3
      IZ=Z/IDIV
      IF(IZ.EQ.0.AND.IMON.EQ.0) GO TO 40
      IMON=1
      ZABCD(II)=DIGITS(IZ+1)
   40 II=II+1
      Z=Z-IDIV*IZ
   50 IDIV=IDIV/10
      IF(A.GT.0) GO TO 60
C-----NATURAL ISOTOPIC MIXTURE.
      ZABCD(9)=NATURE(1)
      ZABCD(10)=NATURE(2)
      ZABCD(11)=NATURE(3)
      ZABCD(12)=NATURE(4)
      GO TO 90
C-----DEFINE A FIRST DIGIT TO LAST.
   60 IDIV=100
      IMON=0
      II=8
      DO 80 I=1,3
      IA=A/IDIV
      IF(IA.EQ.0.AND.IMON.EQ.0) GO TO 70
      IMON=1
      II=II+1
      ZABCD(II)=DIGITS(IA+1)
   70 A=A-IDIV*IA
   80 IDIV=IDIV/10
   90 RETURN
      END
      SUBROUTINE SUB4(ISHELL,SHBCD)
C=======================================================================
C
C     SHORT FORM OF SUBSHELL I.D.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*1 SHBCD,BLANK
      DIMENSION SHBCD(12)
      DATA BLANK/' '/
      CALL SUBHOL(ISHELL,SHBCD)
      DO 10 I=1,12
      IF(SHBCD(I).EQ.BLANK) GO TO 20
   10 CONTINUE
      RETURN
   20 DO 30 K=I,12
   30 SHBCD(K)=BLANK
      RETURN
      END
      SUBROUTINE SUBHOL(ISHELL,SHBCD)
C=======================================================================
C
C     USE ENDL SUBSHELL DESIGNATOR TO DEFINE SUBSHELL
C
C     ARGUMENTS
C     =========
C     ISHELL  = ENDL SUBSHELL DESIGNATOR
C     SHBCD   = HOLLERITH EQUIVALENT RETURNED IN 3A4 FORMAT
C
C     FOR EXAMPLE, FOR ISHELL=1, SHBCD IS RETURNED AS 'K (1S1/2)   '
C     WITHOUT THE ENCLOSING QUOTES.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      CHARACTER*4 SHTAB,SHTAB1,SHTAB2,SHTAB3,SHBCD,TOTAL
      DIMENSION SHTAB(3,62),SHTAB1(3,27),SHTAB2(3,27),SHTAB3(3,8),
     1 SHBCD(3),TOTAL(3)
      EQUIVALENCE (SHTAB(1,1),SHTAB1(1,1)),(SHTAB(1,28),SHTAB2(1,1)),
     1 (SHTAB(1,55),SHTAB3(1,1))
      DATA SHTAB1/
     1 'K (1','s1/2',')   ','L (2',')   ','    ','L1 (','2s1/','2)  ',
     2 'L23 ','(2p)','    ','L2 (','2p1/','2)  ','L3 (','2p3/','2)  ',
     3 'M (3',')   ','    ','M1 (','3s1/','2)  ','M23 ','(3p)','    ',
     4 'M2 (','3p1/','2)  ','M3 (','3p3/','2)  ','M45 ','(3d)','    ',
     5 'M4 (','3d3/','2)  ','M5 (','3d5/','2)  ','N (4',')   ','    ',
     6 'N1 (','4s1/','2)  ','N23 ','(4p)','    ','N2 (','4p1/','2)  ',
     7 'N3 (','4p3/','2)  ','N45 ','(4d)','    ','N4 (','4d3/','2)  ',
     8 'N5 (','4d5/','2)  ','N67 ','(4f)','    ','N6 (','4f5/','2)  ',
     9 'N7 (','4f7/','2)  ','O (5',')   ','    ','O1 (','5s1/','2)  '/
      DATA SHTAB2/
     1 'O23 ','(5p)','    ','O2 (','5p1/','2)  ','O3 (','5p3/','2)  ',
     2 'O45 ','(5d)','    ','O4 (','5d3/','2)  ','O5 (','5d5/','2)  ',
     3 'O67 ','(5f)','    ','O6 (','5f5/','2)  ','O7 (','5f7/','2)  ',
     4 'O89 ','(5g)','    ','O8 (','5g7/','2)  ','O9 (','5g9/','2)  ',
     5 'P (6',')   ','    ','P1 (','6s1/','2)  ','P23 ','(6p)','    ',
     6 'P2 (','6p1/','2)  ','P3 (','6p3/','2)  ','P45 ','(6d)','    ',
     7 'P4 (','6d3/','2)  ','P5 (','6d5/','2)  ','P67 ','(6f)','    ',
     8 'P6 (','6f5/','2)  ','P7 (','6f7/','2)  ','P89 ','(6g)','    ',
     9 'P8 (','6g7/','2)  ','P9 (','6g9/','2)  ','P101','1 (6','h)  '/
      DATA SHTAB3/
     1 'P10 ','(6h9','/2) ','P11 ','(6h1','1/2)','Q (7',')   ','    ',
     2 'Q1 (','7s1/','2)  ','Q23 ','(7p)','    ','Q2 (','7p1/','2)  ',
     3 'Q3 (','7p3/','2)  ','ERRO','R   ','    '/
      DATA TOTAL/'Tota','l   ','   '/
C-----INSURE SUBSHELL INDEX IS IN LEGAL RANGE (1 TO 61).
      KSHELL=ISHELL
      IF(KSHELL.NE.0) GO TO 20
      DO 10 I=1,3
   10 SHBCD(I)=TOTAL(I)
      RETURN
   20 IF(KSHELL.LT.1.OR.KSHELL.GT.61) KSHELL=62
C-----DEFINE SUBSHELL.
      DO 30 I=1,3
   30 SHBCD(I)=SHTAB(I,KSHELL)
      RETURN
      END
      FUNCTION ISDOWN(I)
C=======================================================================
C
C     COLLAPSE SUBSHELL INDICES TO MINIMUM SIZE.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      DIMENSION ISTAB(61)
      DATA ISTAB/
     1  1, 0, 2, 0, 3,
     2  4, 0, 5, 0, 6,
     3  7, 0, 8, 9, 0,
     4 10, 0,11,12, 0,
     5 13,14, 0,15,16,
     6  0,17, 0,18,19,
     7  0,20,21, 0,22,
     8 23, 0,24,25, 0,
     9 26, 0,27,28, 0,
     A 29,30, 0,31,32,
     B  0,33,34, 0,35,
     C 36, 0,37, 0,38,
     D 39/
      ISDOWN=ISTAB(I)
      RETURN
      END
      FUNCTION ISUP(I)
C=======================================================================
C
C     EXPAND SUBSHELL INDICE TO ORIGINAL SIZE.
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      DIMENSION ISTAB(39)
      DATA ISTAB/
     1  1, 3, 5, 6, 8,
     2 10,11,13,14,16,
     3 18,19,21,22,24,
     4 25,27,29,30,32,
     5 33,35,36,38,39,
     6 41,43,44,46,47,
     7 49,50,52,53,55,
     8 56,58,60,61/
      ISUP=ISTAB(I)
      RETURN
      END
      SUBROUTINE RANGER(Z,IZ1,IZ2,IZMAX)
C=======================================================================
C
C     DEFINE NON-ZERO RANGE OF VS. Z DATA
C
C=======================================================================
      IMPLICIT REAL*8 (A-H,O-Z)
      IMPLICIT INTEGER*4 (I-N)
      SAVE
      DIMENSION Z(IZMAX)
      DATA ZERO/0.0D+00/
      DO 10 IZ1=1,IZMAX
      IF(Z(IZ1).NE.ZERO) GO TO 20
   10 CONTINUE
      IZ1=0
      IZ2=0
      RETURN
   20 DO 30 IZ2=IZMAX,IZ1,-1
      IF(Z(IZ2).NE.ZERO) GO TO 40
   30 CONTINUE
      IZ1=0
      IZ2=0
   40 RETURN
      END
      SUBROUTINE OUT9(ZIN,FIELD)
C=======================================================================
C
C     PURPOSE
C     =======
C     FORMAT NUMBER FOR OUTPUT TO INCLUDE AS MANY DIGITS OF
C     ACCURACY AS POSSIBLE.
C     12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C                  No test for 4 or more digit exponent
C
C     10/25/2014 - Changed from D to E exponential form in OUTPUT.
C
C     040923 - CHANGED ZLOW FROM 1.0E-4 TO 1.0E-3
C              NEAR 1.0E-3 PRECISION GOES FROM 999,999 TO 1,000,000
C              FOR SMOOTHEST TRANSITION FROM 6 TO 7 DIGITS
C
C     ARGUMENTS
C     =========
C     Z        = FLOATING POINT NUMBER OF BE OUTPUT (INPUT)
C     FIELD    = 11A1 CHARACTERS TO OUTPUT          (OUTPUT)
C
C     METHOD
C     ======
C     COLUMNS            12345678901     ACCURACY
C     -------------------------------------------
C     0 TO 10^-9          1.2345E-12     5 DIGITS
C     10^-9 TO 10^-3      1.23456E-8     6 DIGITS
C     10^-3 TO 10^-2      .001234567     7 DIGITS
C     10^-2 TO 10^-1      .012345678     8 DIGITS
C     10^-1 TO 1          .123456789     9 DIGITS
C     1 TO 10^9           12345.6789     9 DIGITS
C     10^9 TO 10^10       1.23456E+9     6 DIGITS
C     10^10 >             1.2345E+12     5 DIGITS
C
C     OUTPUT WILL BE IN 11 COLUMN FORMAT
C
C     WARNING - THIS IS NOT A GENERAL ROUNDING ROUTINE WHICH WILL WORK
C               FOR ROUNDING TO ANY NUMBER OF DIGITS - IT WILL ONLY
C               WORK PROPERLY FOR THE RANGES INDICATED ABOVE, FOR
C               11 COLUMN OUTPUT.
C
C=======================================================================
      implicit real*8 (a-h,o-z)
      implicit integer*4 (i-n)
      save
c-----INTEGER*8 for long integers
      integer*8 INMANT,INMAN9,INMAN8,INMAN7,INMAN6,INMAN5,INMAN4
      CHARACTER*1 FIELD,DIGITS,ZEROH
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
      DIMENSION DIGITS(0:9),FIELD(11),ZEROH(11),
     1 TENS(-99:99),ROUNDER(-99:99)
      DATA DIGITS/
     1 '0','1','2','3','4','5','6','7','8','9'/
C-----RETURN FOR = 0
      DATA ZEROH/
     1 ' ','0','.','0',' ',' ',' ',' ',' ',' ',' '/
C-----LOWER TRANSITION POINT FROM 7 TO 9 DIGIT OUTPUT
      DATA ZLOW/1.0D-03/
C-----UPPER TRANSITION POINT FROM 9 TO 7 DIGIT OUTPUT
      DATA ZHIGH/1.0D+09/
      DATA TENTH/1.0D-01/
c-----------------------------------------------------------------------
c
c     ON FIRST CALL INITIALIZE POWERS OF 10
c
c-----------------------------------------------------------------------
      DATA IPASS/0/
      IF(IPASS.NE.0) GO TO 50
      IPASS=1
c-----INTEGER*8 for long integers
      INMAN4  = 10000
      INMAN5  = 10*INMAN4
      INMAN6  = 10*INMAN5
      INMAN7  = 10*INMAN6
      INMAN8  = 10*INMAN7
      INMAN9  = 10*INMAN8
      TENS(0)=1.0D+00
C-----12/18/2012 - EXTENDED FOR 3 DIGIT EXPONENT
C-----11/22/2013 - Restricted to 2 Digit Exponent
      DO 10 I=1,99
      TENS( I)=TENS(I-1)*10.0D+00
      TENS(-I)=TENS(1-I)/10.0D+00
   10 ROUNDER(I)  = 5.001D-05
      DO 20 I=-99,-10
   20 ROUNDER(I)  = 5.001D-05
      DO 30 I=-9,-4
   30 ROUNDER(I)  = 5.001D-06
      ROUNDER(-3) = 5.001D-07
      ROUNDER(-2) = 5.001D-08
      ROUNDER(-1) = 5.001D-09
      ROUNDER( 0) = 5.001D-09
      DO 40 I=1,8
   40 ROUNDER(I)  = 5.001D-09
      ROUNDER(9)  = 5.001D-06
c-----------------------------------------------------------------------
C
C     IMMEDIATELY RETURN 0.00000+00.
C
c-----------------------------------------------------------------------
C-----02/14/04 - ADDED, JUST IN CASE
   50 IF(DABS(ZIN).le.0.0D+0) GO TO 60
      IF(DABS(ZIN).lt.1.0D-99) GO TO 60    ! Only 2 digit exponents
      IF(DABS(ZIN).gt.1.0D+99) GO TO 60
c----- ZIN = 0 handled above
      IF(ZIN.lt.0.0d+0) go to 80
      go to 90
c-----Return 0
   60 DO 70 I=1,11
   70 FIELD(I)=ZEROH(I)
      RETURN
c-----------------------------------------------------------------------
C
C     DEFINE SIGN OF MANTISSA AND ABSOLUTE MANTISSA
C
c-----------------------------------------------------------------------
C-----NEGATIVE.
   80 FIELD(1)='-'
      Z=-ZIN
      GO TO 100
C-----POSITIVE.
   90 FIELD(1)=' '
      Z=ZIN
c-----------------------------------------------------------------------
c
c     DEFINE EXPONENT AND NORMALIZED MANTISSA
c
c-----------------------------------------------------------------------
  100 IEXP=DLOG10(Z)
      IF(Z.LT.1.0D+00) IEXP = IEXP - 1
c-----11/22/2013 - Decide here on F or D Output
c-----11/26/2013 - Restrict to 2 digit exponent - may change by 1 or 2
      if(IABS(IEXP).gt.97) go to 60
      if(ZIN.lt.0.0D+0) then
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      else
      if(Z.LE.ZLOW.OR.Z.GE.ZHIGH) then
      ZN=Z*TENS(-IEXP) + TENTH*ROUNDER(IEXP) ! Extra Digit
      else
      ZN=Z*TENS(-IEXP) + ROUNDER(IEXP)       ! Standard
      endif
      endif
      IF(ZN.eq.1.0D+00) go to 120            ! Test rounding underflow
      IF(ZN.gt.1.0D+00) go to 110            ! Test rounding underflow
      IEXP=IEXP-1         ! MUST be < 1
      GO TO 120
  110 IF(ZN.lt.10.0D+00) go to 120           ! Test rounding overflow
      IEXP=IEXP+1         ! MUST be >= 10
      ZN=ZN/10.0D+00
  120 Z = ZN*TENS(IEXP)
c-----------------------------------------------------------------------
C
C     SELECT F OR D FORMAT
C
c-----------------------------------------------------------------------
      IF(Z.LE.ZLOW.OR.Z.GE.ZHIGH) GO TO 150
c-----------------------------------------------------------------------
C
C     F FORMAT
C
C     12345678901
C      X.XXXXXXXX = 9 DIGITS
C      .001234567
C      123456789.
C
c-----------------------------------------------------------------------
C-----DEFINE 6 TO 9 DIGIT MANTISSA WITH ROUNDING
      IPOWER=8-IEXP
      IF(IEXP.LT.0) IPOWER=8
      INMANT=Z*TENS(IPOWER)
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      if(INMANT.lt.INMAN9) go to 130
      INMANT=INMAN8
      IEXP=IEXP+1
C-----DECIMAL POINT.
  130 IDOT=3+IEXP
      IF(IDOT.LE.2) THEN
C----- IF < 1, MOVE DECIMAL POINT TO COLUMN 2 AND ADD A DIGIT
      IDOT=2
      INMANT=Z*10.0D+00*TENS(IPOWER)
      ENDIF
      FIELD(IDOT)='.'
C-----MANTISSA - LAST DIGIT TO FIRST.
      II=11
      DO 140 I=2,11
      IF(II.EQ.IDOT) GO TO 140
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  140 II=II-1
      RETURN
c-----------------------------------------------------------------------
C
C     D FORMAT
C
C     12345678901
C      X.XXXXD+NN = 5 DIGITS
C      X.XXXXXD+N = 6 DIGITS
C
C     11/22/2013 - If not negative, use first column
C     12345678901
C     X.XXXXXE+NN = 6 DIGITS
C     X.XXXXXXE+N = 7 DIGITS
c
c-----------------------------------------------------------------------
C-----Negative?
  150 IF(ZIN.lt.0.0d+0) go to 170
c
c     POsitive. Use first column - Decimal point is always in column 2
c
      FIELD(2)='.'
      KDOT    = 2
      ISTART  = 1
      IF(IABS(IEXP).GE.10) GO TO 160
      ID=8                               ! 1 Digit exponent
      INMANT=(1.0D+06)*ZN
      IF(INMANT.lt.INMAN7) go to 190
      INMANT=INMAN6
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 190
  160 ID=7                               ! 2 Digit exponent
      INMANT=(1.0D+05)*ZN
C-----CHECK FOR OVERFLOW DUE TO ROUNDING
      IF(INMANT.lt.INMAN6) go to 190
      INMANT=INMAN5
      IEXP=IEXP+1
c
c     Negative Number - Cannot use first column
c
C-----DECIMAL POINT IS ALWAYS IN COLUMN 3
  170 FIELD(3)='.'
      KDOT    = 3
      ISTART  = 2
      IF(IABS(IEXP).GE.10) GO TO 180
      ID=8                                 ! 1 Digit Exponent
      INMANT=(1.0D+05)*ZN
      IF(INMANT.lt.INMAN6) go to 190
      INMANT=INMAN5
      IEXP=IEXP+1
      IF(IABS(IEXP).LT.10) GO TO 190
  180 ID=7                                 ! 2 Digit Exponent
      INMANT=(1.0D+04)*ZN
      IF(INMANT.lt.INMAN5) go to 190
      INMANT=INMAN4
      IEXP=IEXP+1
C
C     DEFINE MANTISSA
C
  190 IEXPS=ID+1
      II=ID
      DO 200 I=ISTART,ID
      IF(II.EQ.KDOT) GO TO 200
      I2=INMANT/10
      I3=INMANT-10*I2
      FIELD(II)=DIGITS(I3)
      INMANT=I2
  200 II=II-1
C
C     E
C
      FIELD(IEXPS) = 'E'
      IEXPS = IEXPS + 1
C
C     SIGN OF EXPONENT
C
      IF(IEXP.ge.0) go to 210
      IEXP=-IEXP
      FIELD(IEXPS)='-'
      GO TO 220
  210 FIELD(IEXPS)='+'
C
C     EXPONENT
C
  220 IF(IEXP.lt.10) go to 230
      KEXP=IEXP/10
      FIELD(10)=DIGITS(KEXP)
      IEXP=MOD(IEXP,10)
  230 FIELD(11)=DIGITS(IEXP)
c
c     If using column 1 but mantissa ends in 0, move mantissa right.
c
      if(KDOT.eq.2.and.FIELD(ID).eq.'0') then
      do k=ID-1,1,-1
      FIELD(k+1)=FIELD(k)
      enddo
      FIELD(1) = ' '
      endif
      RETURN
      END
